﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using TotallyRadEngine10;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework;
using PolygonCollisionLibrary;
using Microsoft.Xna.Framework.Graphics;

namespace ProjectMaternity
{
    public class Bed : Interactable
    {
        // Patient in the bed, null if none
        private Patient mPatient;
        private string mCurrentState;

        // Nearest cabinets
        public Cabinet NearestCabMed { get; set; }
        public Cabinet NearestCabLaundry { get; set; }

        public string CurrentState 
        {
            get
            {
                return mCurrentState;
            }
            set
            {
                mCurrentState = value;
            }
        }
        public string PatientCurrentState
        {
            get
            {
                string state = "";
                if (mPatient != null)
                {
                    state = mPatient.CurrentState;
                }

                return state;
            }
        }
        public float PatientUrgencyTimer
        {
            get
            {
                float urgency = -1;
                if (mPatient != null)
                {
                    urgency = mPatient.UrgencyTimer;
                }

                return urgency;
            }
        }

        public Bed()
            : base()
        {
            // Proper collision
            Vector2[] colVerts = new Vector2[4] {new Vector2(1.0f, 2.0f),
                                                             new Vector2(-1.0f, 2.0f),
                                                             new Vector2(-1.0f, -2.0f),
                                                             new Vector2(1.0f, -2.0f)};
            BoundingPolygon = new Polygon(colVerts, true);

            mCurrentState = "Clean";    // Initial bed state
        }

        protected override bool OnInteraction(Interactable other)
        {
            bool success = true;

            if (other is Nurse)
            {
                // If the bed is empty and clean and the player is being followed by a patient, put the patient in the bed
                if (mPatient == null)
                {
                    if (mCurrentState == "Clean")
                    {
                        if (other.IsInteractingWith<Patient>())
                        {
                            mPatient = (Patient)other.GetInteractable<Patient>();

                            // Lay in bed PREGNANT WOMAN!  
                            //mPatient.Model = ContentCache.GetModel("Models\\Animated\\LadyLie");
                            mPatient.SetAnimation("Lie", 0.0f, 1.4f);

                            //mPatient.Rotation.X -= 90;
                            mPatient.Rotation.Y = (Rotation.Y);

                            mPatient.Colour = Color.White;

                            // !WARNING! Only works if beds are horizontal or vertical
                            if (Rotation.Y == 0 || Rotation.Y == 360)
                                mPatient.Position = this.Position + new Vector3(0.0f, -0.0f, -0.3f);
                            else if (Rotation.Y == 180)
                                mPatient.Position = this.Position + new Vector3(0.0f, -0.0f, +0.3f);
                            else if (Rotation.Y == 90)
                                mPatient.Position = this.Position + new Vector3(-0.3f, -0.0f, 0.0f);
                            else if (Rotation.Y == 270)
                                mPatient.Position = this.Position + new Vector3(+0.3f, -0.0f, 0.0f);

                            // Stop the patient following the player
                            mPatient.StopInteractionWith<Nurse>();

                            // Disable chair-finding PathFinder
                            mPatient.MacroPathFinder.CancelCurrentPath();
                            mPatient.MicroPathFinder.CancelCurrentPath();

                            // Patient is temporarily not urgent enough to require attention when first put in bed
                            mPatient.UrgencyTimer = -1;

                            InteractWith(mPatient);
                            BeginInteraction(mPatient, -1);

                            // Could add in a timed interaction with the player here if we want for animations or whatevs  
                            
                        }
                    }

                    else  // Clean dirty bed if the player is not being followed
                    {
                        if (!other.IsInteractingWith<Patient>())
                        {
                            if ((other as Nurse).CurrentItem == Item.CleanSheets)
                            {
                                //other.Model = ContentCache.GetModel("Models\\Animated\\NurseAssist");
                                BeginInteraction(other, 1);                                
                                (other as Nurse).CurrentItem = Item.DirtySheets;
                                other.SetAnimation("Assist", 0.0f, 1.0f);
                                AudioPlayer.PlaySound("CleanBed");
                            }
                        }
                    }
                }

                else // There is a patient in this bed, treat them if they need it
                {
                    if (!(other as Nurse).IsInteractingWith<Patient>())
                    {
                        if (mPatient.CurrentState == "Complicated")
                        {
                            if ((other as Nurse).CurrentItem == Item.Medical)
                            {
                                BeginInteraction(other, 2);
                                (other as Nurse).CurrentItem = Item.None;
                                other.SetAnimation("Assist", 0.0f, 0.5f);
                            }
                        }
                        else if (mPatient.CurrentState == "Delivering")
                        {
                            BeginInteraction(other, 2);
                            other.SetAnimation("Assist", 0.0f, 0.5f);
                        }
                    }
                }
            }

            return success;
        }

        public override void Continue(GameTime gameTime)
        {
            // Remove patient from bed after they've finished resting
            if (mPatient != null)
            {
                if (mPatient.CurrentState == "Leaving")
                {
                    mPatient = null;
                }
            }

            base.Continue(gameTime);
        }

        public override void InteractionComplete(Interaction interaction)
        {
            // After the delayed interaction with the player is over, treat the patient
            if (interaction.Other is Nurse)
            {
                if (mPatient != null)
                {
                    if (mPatient.CurrentState == "Delivering")
                    {
                        // Random chance of a complication based on urgency level
                        Random random = new Random();
                        if (random.Next(0, 100) > 100 - (PlayState.ComplicationChance))
                        {
                            // Give the player a little bit more time
                            mPatient.UrgencyTimer -= 2;
                            mPatient.CurrentState = "Complicated";
                            AudioPlayer.PlaySound("WomanComplication");

                            PlayState.BirthComplications++;
                        }
                        else
                        {
                            mPatient.CurrentState = "Resting";
                            PlayState.TreatedPatientCount++;
                            mPatient.UrgencyTimer = -1;
                            mPatient.Delay = 5;
                        }
                    }
                    else if (mPatient.CurrentState == "Complicated")
                    {
                        mPatient.CurrentState = "Resting";
                        PlayState.TreatedPatientCount++;
                        mPatient.UrgencyTimer = -1;
                        mPatient.Delay = 5;
                    }
                }
                else
                {
                    if (mCurrentState == "Dirty")
                    {
                        //interaction.Other.Model = ContentCache.GetModel("Models\\Animated\\NurseIdle");
                        this.mCurrentState = "Clean";
                    }
                }
            }
            
        }

        public override void Update(GameTime gameTime)
        {
            Continue(gameTime);

            // Hack to prevent patients being counted as in bed after leaving/death
            if (mCurrentState == "Dirty" && mPatient != null)
            {
                StopInteractionWith<Patient>();
                mPatient = null;
            }

            base.Update(gameTime);
        }
    }
}
