﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using TotallyRadEngine10;
using Microsoft.Xna.Framework;

namespace ProjectMaternity
{
    class BubblesUI : UI
    {
        public Matrix ViewMatrix;
        public Matrix ProjectionMatrix;
        public List<Patient> Patients = null;
        private int LastPatientCount;

        public BubblesUI()
            : base()
        {
            LastPatientCount = 0;
        }

        public override void Update()
        {
            if (Patients != null)
            {
                for (int i = 0; i < Patients.Count; i++)
                {
                    Patient patient = Patients[i];

                    if (i >= LastPatientCount)
                    {
                        if (patient.UrgencyTimer != -1 && patient.CurrentState != "Leaving")
                        {
                            AddImage("Bubble" + patient.UniqueID.ToString(), new Image(Vector2.Zero,
                                     0.2f, ContentCache.GetTexture("Sprites\\BedBubble"),
                                     AnimationType.None, Alignment.Centre, patient.UrgencyColor));
                        }
                    }

                    if (patient.UrgencyTimer != -1)
                    {
                        Vector3 bubblePos3D = new Vector3(patient.Position.X, patient.Position.Y + 6.0f, patient.Position.Z);
                        Vector4 transPos = new Vector4(bubblePos3D, 1.0f);
                        transPos = Vector4.Transform(transPos, ViewMatrix);
                        transPos = Vector4.Transform(transPos, ProjectionMatrix);
                        transPos /= transPos.W;

                        transPos.X += 1;
                        transPos.Y += 1;
                        transPos.X /= 2;
                        transPos.Y /= 2;
                        transPos.X *= Renderer.BackBufferWidth;
                        transPos.Y *= Renderer.BackBufferHeight;
                        transPos.Y = Renderer.BackBufferHeight - transPos.Y; // Not entirley sure why this is necesarry   

                        try
                        {
                            GetImage("Bubble" + patient.UniqueID.ToString()).Color.A = 255;
                            GetImage("Bubble" + patient.UniqueID.ToString()).Position = new Vector2(transPos.X + 50, transPos.Y - 30);

                            if (!patient.IsInteractingWith<Bed>())
                            {
                                GetImage("Bubble" + patient.UniqueID.ToString()).Sprite = ContentCache.GetTexture("Sprites\\BedBubble");
                                GetImage("Bubble" + patient.UniqueID.ToString()).Color = patient.UrgencyColor;
                            }
                            else
                            {
                                if (patient.CurrentState == "Delivering")
                                {
                                    GetImage("Bubble" + patient.UniqueID.ToString()).Sprite = ContentCache.GetTexture("Sprites\\BirthBubble");
                                    GetImage("Bubble" + patient.UniqueID.ToString()).Color = patient.UrgencyColor;
                                }
                                else if (patient.CurrentState == "Complicated")
                                {
                                    GetImage("Bubble" + patient.UniqueID.ToString()).Sprite = ContentCache.GetTexture("Sprites\\ComplicationBubble");
                                    GetImage("Bubble" + patient.UniqueID.ToString()).Color = patient.UrgencyColor;
                                }
                            }
                        }
                        catch
                        {
                        }
                    }
                    else
                    {
                        try
                        {
                            GetImage("Bubble" + patient.UniqueID.ToString()).Color.A = 0;
                        }
                        catch { }
                    }
                }

                LastPatientCount = Patients.Count;
            }

            base.Update();
        }
    }
}
