﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using TotallyRadEngine10;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework;
using PolygonCollisionLibrary;

namespace ProjectMaternity
{
    public enum CabinetType
    {
        Medicine,
        Laundry
    }

    public class Cabinet : Interactable
    {
        private CabinetType mCabinetType;

        public CabinetType CabinetType
        {
            get
            {
                return mCabinetType;
            }
        }

        public Cabinet(CabinetType cabinetType)
            : base()
        {
            Rotation.Y = 225;
            this.mCabinetType = cabinetType;

            if (mCabinetType == CabinetType.Laundry)
            {
                Vector2[] colVerts = new Vector2[4] {new Vector2(0.7f, 2.0f),
                                                             new Vector2(-0.7f, 2.0f),
                                                             new Vector2(-0.7f, -2.0f),
                                                             new Vector2(0.7f, -2.0f)};
                BoundingPolygon = new Polygon(colVerts, true);
            }
            else if (mCabinetType == CabinetType.Medicine)
            {
                Vector2[] colVerts = new Vector2[4] {new Vector2(1.0f, 1.0f),
                                                             new Vector2(-1.0f, 1.0f),
                                                             new Vector2(-1.0f, -1.0f),
                                                             new Vector2(1.0f, -1.0f)};
                BoundingPolygon = new Polygon(colVerts, true);
            }
        }

        protected override bool OnInteraction(Interactable other)
        {
            bool success = true;

            if (other is Nurse)
            {
                Patient playerPatient = (Patient)other.GetInteractable<Patient>();

                // Don't let player get items when being followed
                if (playerPatient == null)
                {
                    if (this.mCabinetType == CabinetType.Medicine)
                    {
                        if ((other as Nurse).CurrentItem == Item.None)
                        {
                            // Pick up medicine
                            (other as Nurse).CurrentItem = Item.Medical;
                        }
                    }
                    
                    if (this.mCabinetType == CabinetType.Laundry)
                    {
                        // Pick up clean sheets
                        if ((other as Nurse).CurrentItem == Item.None)
                        {
                            (other as Nurse).CurrentItem = Item.CleanSheets;
                        }

                        // Put back dirty sheets
                        else if ((other as Nurse).CurrentItem == Item.DirtySheets)
                        {
                            (other as Nurse).CurrentItem = Item.None;
                        }

                        // Put back extra clean sheets if this is an AINurse
                        else if (other is AINurse && (other as Nurse).CurrentItem == Item.CleanSheets)
                        {
                            (other as Nurse).CurrentItem = Item.None;
                        }
                    }
                }
            }

            return success;
        }

        public override void Continue(GameTime gameTime)
        {
            base.Continue(gameTime);
        }

        public override void InteractionComplete(Interaction interaction)
        {
        }

        public override void Update(GameTime gameTime)
        {
            Continue(gameTime);

            base.Update(gameTime);
        }
    }
}
