﻿
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using TotallyRadEngine10;

namespace ProjectMaternity
{
    public static class CollisionManager
    {
        /// <summary>
        /// Separates two interactables.
        /// inter2 will be "pushed" away from inter1.
        /// </summary>
        /// <param name="inter">First interactable</param>
        /// <param name="inter2">Second interactable</param>
        public static void Resolve(Interactable inter1, Interactable inter2)
        {
                // Is the player colliding with the interactable? If so, separate them
                if (inter2.BoundingPolygon.Intersects(ref inter1.BoundingPolygon))
                {
                    inter2.Position += new Vector3(
                        inter2.BoundingPolygon.collisionData.MTD.X,
                        0.0f,
                        inter2.BoundingPolygon.collisionData.MTD.Y);
                }
        }

        /// <summary>
        /// Separates a list of Interactables from an Interactable.
        /// The InteractableList will be "pushed" away from the Interactable.
        /// </summary>
        /// <param name="interactable">Interactable to separate from</param>
        /// <param name="interList">List of Interactables</param>
        public static void ResolveList<T>(Interactable interactable, List<T> interList)
        {
            for (int i = 0; i < interList.Count; i++)
            {
                Interactable inter2 = interList[i] as Interactable;

                // Is the player colliding with the interactable? If so, separate them
                if (inter2.BoundingPolygon.Intersects(ref interactable.BoundingPolygon))
                {
                    inter2.Position += new Vector3(
                        inter2.BoundingPolygon.collisionData.MTD.X,
                        0.0f,
                        inter2.BoundingPolygon.collisionData.MTD.Y);
                }
            }
        }
    }
}
