﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using TotallyRadEngine10;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace ProjectMaternity
{
    class EndLevelState : State
    {
        int mNextLevelNo;
        bool mFailure;

        public override void Load(PersistentData persistent)
        {
            mFailure = persistent.Booleans["Failure"];

            if (!(persistent.Booleans["Failure"]))
            {
                mNextLevelNo = persistent.Integers["LevelNo"] + 1;

                mUI.Add("Stats", new StatsUI());

                (mUI["Stats"] as StatsUI).TreatedPatients = persistent.Integers["PatientsTreated"];
                (mUI["Stats"] as StatsUI).Complications = persistent.Integers["ComplicationCount"];
                (mUI["Stats"] as StatsUI).Improvement = persistent.Integers["Improved"];
                //(mUI["Stats"] as StatsUI).InfantDeaths = persistent.Integers["InfantDeaths"];
                //(mUI["Stats"] as StatsUI).MaternalDeaths = persistent.Integers["MaternalDeaths"];

            }
            else
            {
                mNextLevelNo = persistent.Integers["LevelNo"];

                mUI.Add("Stats", new StatsUIFail());
                (mUI["Stats"] as StatsUIFail).TreatedPatients = persistent.Integers["PatientsTreated"];
                (mUI["Stats"] as StatsUIFail).Objective = persistent.Integers["Objective"];

            }

            AudioPlayer.StopAll();
            AudioPlayer.PlaySound("EndLevelMusic");
        }

        public override void Update(Microsoft.Xna.Framework.GameTime gameTime)
        {
            if (InputManager.HasButtonBeenPressed(Keys.Enter) ||
                InputManager.HasButtonBeenPressed(Buttons.A))
            {
                if (mNextLevelNo == 0)
                {
                    mNextLevelNo = 0;
                    StateManager.SetState(new PlayState());
                }
                else if (mNextLevelNo >= 1)
                {
                    // Move to the map screen
                    MapState mapState = new MapState();
                    StateManager.SetState(mapState);
                }               
            }

            // Update the UI
            foreach (KeyValuePair<string, UI> ui in mUI)
            {
                ui.Value.Update();
            }            
        }
      
        public override void Draw(Renderer renderer, GameTime gameTime)
        {
            // Draw the UI
            foreach (KeyValuePair<string, UI> ui in mUI)
            {
                ui.Value.Draw(renderer.SpriteBatch);
            }
        }

        public override void Unload(PersistentData persistent)
        {
            persistent.Integers["LevelNo"] = mNextLevelNo;
        }

     
    }
}
