using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
using TotallyRadEngine10;

namespace ProjectMaternity
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        Renderer mRenderer;
        StateManager mStateManager;

        public Game()
        {
            graphics = new GraphicsDeviceManager(this);
            graphics.PreferredBackBufferWidth = 1280;
            graphics.PreferredBackBufferHeight = 1024;
            //graphics.PreferMultiSampling = true;            
            Content.RootDirectory = "Content";
        }

        protected override void Initialize()
        {
            graphics.PreferredBackBufferWidth = graphics.GraphicsDevice.DisplayMode.Width;
            graphics.PreferredBackBufferHeight = graphics.GraphicsDevice.DisplayMode.Height;
            //graphics.ToggleFullScreen();

            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            ContentCache.Init();
            ContentCache.Content = Content;

            mStateManager = new StateManager();
            mRenderer = new Renderer(graphics.GraphicsDevice, spriteBatch);

            AudioPlayer.Initialise();
            //AudioPlayer.PlaySound("DumbAssMusic");

            StateManager.SetState(new IntroState());           

            base.Initialize();
        }

        protected override void LoadContent()
        { 
        }


        protected override void UnloadContent()
        {
        }

        protected override void Update(GameTime gameTime)
        {
            InputManager.UpdateInput();

            // Allows the game to exit
            if (mStateManager.Run(gameTime))
            {
                this.Exit();
            }

            base.Update(gameTime);
        }

        protected override void Draw(GameTime gameTime)
        {
            mRenderer.Clear();

            mStateManager.CurrentState.Draw(mRenderer, gameTime);

            base.Draw(gameTime);
        }
    }
}
