﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using TotallyRadEngine10;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace ProjectMaternity
{
    class GameplayUI : UI
    {
        public TimeSpan Timer;
        public int PatientsTreated;
        public int ComplicationCount;
        //public float InfantMortality;
        public float MaternalMortality;
        public float InitialMortality;
        
        public bool OnCallAvailable;
        public bool OnCallShiftActive;
        public float OnCallTimer;
        public float MaxOnCallTimer;
        public int OnCallLevel;
        public float ShiftTimer;
        public float MaxShiftTimer;

        public int PatientsDeceased = 0;
        public int ObjectiveRate = 0;

        int mOldMarkerX;

        public GameplayUI() : base()
        {
            Vector2 shiftBoxPos = new Vector2(Renderer.BackBufferWidth / 2, 120);

            Text timerTxt = new Text(Timer.Minutes.ToString() + ":" + (Timer.Seconds < 10 ? "0" : "") + Timer.Seconds.ToString(), ContentCache.GetFont("Fonts\\UI"), Color.White, new Vector2(Renderer.BackBufferWidth / 2, 45), 1.0f, AnimationType.None, Alignment.Centre);
            AddText("TimerClock", timerTxt);

            AddImage("OrbOneG", new Image(shiftBoxPos + new Vector2(-65, 18), 1.0f, ContentCache.GetTexture("Sprites\\CrossOrbGold"), AnimationType.Pulsate, Alignment.Centre, new Color(255, 255, 255, 0)));
            AddImage("OrbThreeG", new Image(shiftBoxPos + new Vector2(0, 18), 1.0f, ContentCache.GetTexture("Sprites\\CrossOrbGold"), AnimationType.Pulsate, Alignment.Centre, new Color(255, 255, 255, 0)));
            AddImage("OrbTwoG", new Image(shiftBoxPos + new Vector2(65, 18), 1.0f, ContentCache.GetTexture("Sprites\\CrossOrbGold"), AnimationType.Pulsate, Alignment.Centre, new Color(255, 255, 255, 0)));

            AddImage("OrbOne", new Image(shiftBoxPos + new Vector2(-65, 18), 1.0f, ContentCache.GetTexture("Sprites\\CrossOrbSilver"), AnimationType.Pulsate, Alignment.Centre, new Color(255, 255, 255, 0)));
            AddImage("OrbThree", new Image(shiftBoxPos + new Vector2(0, 18), 1.0f, ContentCache.GetTexture("Sprites\\CrossOrbSilver"), AnimationType.Pulsate, Alignment.Centre, new Color(255, 255, 255, 0)));
            AddImage("OrbTwo", new Image(shiftBoxPos + new Vector2(65, 18), 1.0f, ContentCache.GetTexture("Sprites\\CrossOrbSilver"), AnimationType.Pulsate, Alignment.Centre, new Color(255, 255, 255, 0)));

            AddText("TimerRemain", new Text("Time Remaining", ContentCache.GetFont("Fonts\\UISmall"), Color.White, new Vector2(Renderer.BackBufferWidth / 2, 25), 1.0f, AnimationType.None, Alignment.Centre));
            AddText("OnCallText", new Text("On call nurses unavailable", ContentCache.GetFont("Fonts\\UISmaller"), Color.White, new Vector2(Renderer.BackBufferWidth / 2, 102), 1.0f, AnimationType.None, Alignment.Centre));
            
            AddImage("ShiftBox", new Image(shiftBoxPos, 1.0f, ContentCache.GetTexture("Sprites\\UITimerBox"), AnimationType.None, Alignment.Centre, Color.LightBlue));
            AddImage("TimerBox", new Image(new Vector2(Renderer.BackBufferWidth / 2, 60), 1.0f, ContentCache.GetTexture("Sprites\\UITimerBox"), AnimationType.None, Alignment.Centre, new Color(72, 141, 228)));

            AddImage("ShiftBar", new Image(new Vector2(Renderer.BackBufferWidth / 2, 138), 1.0f, ContentCache.GetTexture("Sprites\\UITimerBox"), AnimationType.None, Alignment.Centre, Color.Red));
            GetImage("ShiftBar").Scale2D.X = 1.0f;
            GetImage("ShiftBar").Scale2D.Y = 0.3f;

            AddImage("Marker", new Image(new Vector2(Renderer.BackBufferWidth / 2, Renderer.BackBufferHeight - 60), 1.0f, ContentCache.GetTexture("Sprites\\Marker"), AnimationType.None, Alignment.Centre));
            AddImage("ProgressBar", new Image(new Vector2(Renderer.BackBufferWidth / 2, Renderer.BackBufferHeight - 60), 1.0f, ContentCache.GetTexture("Sprites\\ProgressBar"), AnimationType.None, Alignment.Centre));            
        }

        public override void Update()
        {
            GetText("TimerClock").String = Timer.Minutes.ToString() + ":" + (Timer.Seconds < 10 ? "0" : "") + Timer.Seconds.ToString();

            DisplayOrbs(); // Show number of available on-call nurses

            if (OnCallAvailable)
            {
                GetImage("ShiftBar").Color = Color.Green;

                if (OnCallShiftActive)
                {
                    GetText("OnCallText").String = "Shift ends in:";
                    GetImage("ShiftBar").Scale2D.X = 1.0f - (ShiftTimer / MaxShiftTimer);
                }
                else if (OnCallTimer < MaxOnCallTimer)
                {
                    GetText("OnCallText").String = "Shift upgrade in:";
                    GetImage("ShiftBar").Scale2D.X = 1.0f - (OnCallTimer / MaxOnCallTimer);
                }
                else
                {
                    GetText("OnCallText").String = "Full shift available!";
                }
            }
            else
            {
                GetImage("ShiftBar").Color = Color.Red;
                GetText("OnCallText").String = "Next shift in:";
                GetImage("ShiftBar").Scale2D.X = 1.0f - (OnCallTimer / MaxOnCallTimer);
            }

            mOldMarkerX = (int)GetImage("Marker").Position.X;
            //int newMarkerX = (int)MathHelper.Clamp((Renderer.BackBufferWidth * 0.5f) +  * (Renderer.BackBufferWidth / (ObjectiveRate * 2))), , (Renderer.BackBufferWidth * 0.5f) + 626);
            float progressPercentage = (float)(((float)PatientsTreated - (float)PatientsDeceased) / (float)ObjectiveRate);
            int newMarkerX = (int)MathHelper.Clamp((((Renderer.BackBufferWidth * 0.5f)) + (626 * progressPercentage)), (Renderer.BackBufferWidth * 0.5f) - 626, (Renderer.BackBufferWidth * 0.5f) + 626); ;

            if(mOldMarkerX < newMarkerX)
            {
                GetImage("Marker").Position.X++;
            }
            else if(mOldMarkerX > newMarkerX)
            {
                GetImage("Marker").Position.X--;
            }

            DisplayOrbs(); // Show number of available on-call nurses

            base.Update();
        }

        private void DisplayOrbs()
        {
            // I know, messy messy! Don't judge me, bitch!
            GetImage("OrbOne").Color.A = GetImage("OrbTwo").Color.A = GetImage("OrbThree").Color.A = 0;
            GetImage("OrbOneG").Color.A = GetImage("OrbTwoG").Color.A = GetImage("OrbThreeG").Color.A = 0;

            if (OnCallLevel == 1) GetImage("OrbOne").Color.A = 255;
            if (OnCallLevel >= 2) GetImage("OrbOneG").Color.A = 255;
            if (OnCallLevel >= 3) GetImage("OrbTwo").Color.A = 255;
            if (OnCallLevel >= 4) GetImage("OrbTwoG").Color.A = 255;
            if (OnCallLevel >= 5) GetImage("OrbThree").Color.A = 255;
            if (OnCallTimer > MaxOnCallTimer) GetImage("OrbThreeG").Color.A = 255;


        }
    }
}
