﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using TotallyRadEngine10;
using System.Reflection;

namespace ProjectMaternity
{
    public class Interaction
    {
        public Interactable Other;
        public float Timer;
    }

    public abstract class Interactable : Entity
    {
        public Nurse TargetedBy { get; set; }
        protected List<Interaction> mInteractions;

        public Interactable() : base()
        {
            mInteractions = new List<Interaction>();
        }

        private object InvokeGenericMethod(string typeName, string methodName, object[] parameters)
        {
           Type type = System.Type.GetType(typeName);

            MethodInfo method = this.GetType().GetMethod(methodName,
                BindingFlags.Public | BindingFlags.Instance);

            MethodInfo genericMethod = method.MakeGenericMethod(type);
            return genericMethod.Invoke(this, parameters);
        }

        public bool InteractWith(Interactable other)
        {
            return other.OnInteraction(this);
        }

        protected abstract bool OnInteraction(Interactable other);
        public virtual void Continue(GameTime gameTime)
        {
            List<Interaction> completeInteractions = new List<Interaction>();
            
            foreach (Interaction interaction in mInteractions)
            {
                if (interaction.Timer != -1)
                {
                    interaction.Timer -= (float)(gameTime.ElapsedGameTime.TotalMilliseconds / 1000);

                    if (interaction.Timer <= 0)
                    {
                        // This interaction is complete, prepare it for stopping
                        completeInteractions.Add(interaction);
                    }
                }
            }

            for(int i = 0; i < completeInteractions.Count; i++)
            {
                InvokeGenericMethod(completeInteractions[i].Other.GetType().FullName, "StopInteractionWith", null);
            }
        }
        public abstract void InteractionComplete(Interaction interaction);
        /// <summary>
        /// Adds a new Interaction to each of the interactables involved in the interaction.
        /// </summary>
        /// <param name="other">The other interactable</param>
        /// <param name="time">-1 = ongoing interaction. Otherwise timer value.</param>
        public void BeginInteraction(Interactable other, int time)
        {
            Interaction newInteraction = new Interaction();
            newInteraction.Other = other;
            newInteraction.Timer = time;

            mInteractions.Add(newInteraction);

            Interaction newInteraction2 = new Interaction();
            newInteraction2.Other = this;
            newInteraction2.Timer = time;

            other.mInteractions.Add(newInteraction2);
        }

        public bool IsInteractingWith<T>()
        {
            bool interactionFound = false;

            foreach (Interaction interaction in mInteractions)
            {
                if (interaction.Other is T)
                {
                    interactionFound = true;
                }
            }

            return interactionFound;
        }

        public Interactable GetInteractable<T>()
        {
            Interactable interactable = null;      

            foreach (Interaction interaction in mInteractions)
            {
                if (interaction.Other is T)
                {
                    interactable = interaction.Other;                   
                }
            }

            if (interactable == null)
            {
                //throw new Exception("There is no interaction ocurring between " + this.GetType().FullName + " and " + T + ". You should check an interaction is ocurring using IsInteractingWith<T> before using this method fool!");
            }

            return interactable;
        }

        public void StopInteractionWith<T>()
        {
            bool interactionFound = false;
            Interaction interaction = null;
            Interaction otherInteraction = null;
            Interactable other = null;

            for (int i = 0; i < mInteractions.Count; i++)
            {
                if (mInteractions[i].Other is T)
                {
                    interactionFound = true;
                    interaction = mInteractions[i];
                    other = mInteractions[i].Other;
                    mInteractions.RemoveAt(i);                    
                }
            }

            if (interactionFound)
            {
                for (int i = 0; i < other.mInteractions.Count; i++)
                {
                    if (other.mInteractions[i].Other.GetType() == this.GetType())
                    {
                        otherInteraction = other.mInteractions[i];
                        other.mInteractions.RemoveAt(i);
                    }
                }

                InteractionComplete(interaction);
                other.InteractionComplete(otherInteraction);
            }
        }

        public override void Update(GameTime gameTime)
        {       
            base.Update(gameTime);
        }
    }
}
