﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using TotallyRadEngine10;
using Microsoft.Xna.Framework;
using PolygonCollisionLibrary;
using Microsoft.Xna.Framework.Graphics;

namespace ProjectMaternity
{
    public class Interactable :  Entity
    {
        // Names of all Interactable states
        protected List<string> mStates;

        // Interactable's current state
        protected string mCurrentState;

        // Collection of Interaction structs accessible by name 
        protected Dictionary<string, Interaction> mInteractions;

        // A reference to the current player
        protected Player mPlayer;

        // Touching the player's InteractionPoly?
        protected bool mTouchingPlayer = false;

        // Colliding with the player's BoundingPoly?
        protected bool mCollidingPlayer = false;

        // The delay following an interaction
        protected float mInteractDelay = 0.0f;

        #region Properties

        public List<string> States
        {
            get
            {
                return mStates;
            }
            set
            {
                mStates = value;
            }
        }

        public string CurrentState
        {
            get
            {
                return mCurrentState;
            }
            set
            {
                mCurrentState = value;
            }
        }

        public Dictionary<string, Interaction> Interactions 
        {
            get
            {
                return mInteractions;
            }
            set
            {
                mInteractions = value;
            }
        }

        public bool TouchingPlayer
        {
            get
            {
                return mTouchingPlayer;
            }
            set
            {
                mTouchingPlayer = value;
            }
        }

        public bool CollidingPlayer
        {
            get
            {
                return mCollidingPlayer;
            }
            set
            {
                mCollidingPlayer = value;
            }
        }

        public float InteractDelay
        {
            get
            {
                return mInteractDelay;
            }
            set
            {
                mInteractDelay = value;
            }
        }

        #endregion

        public Interactable(Player player)
        {
            // Store a reference to the player
            this.mPlayer = player;

            ////////// Set Up States //////////

            this.mStates = new List<string>() { "NonState" };
            this.mCurrentState = mStates[0];

            ////////// Set Up Interactions //////////

            this.mInteractions = new Dictionary<string, Interaction>();
        }

        public override void Update(GameTime gameTime)
        {
            #region Change BoundingPolygon color based on collision with player

            if (mCollidingPlayer)
            {
                this.BoundingPolygon.RenderDataColor = Color.Red;
            }
            else if (mTouchingPlayer)
            {
                this.BoundingPolygon.RenderDataColor = Color.Yellow;
            }
            else
            {
                this.BoundingPolygon.RenderDataColor = Color.White;
            }

            #endregion

            base.Update(gameTime);
        }

        public virtual void PerformInteraction(string interaction){}
    }
}
