﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using TotallyRadEngine10;
using Microsoft.Xna.Framework.Input;

namespace ProjectMaternity
{
    /// <summary>
    /// Maintains/updates a list of Interactables.
    /// Initiates relevant Interactions.
    /// </summary>
    class InteractionManager
    {
        // List of all Interactables in the current state
        private List<Interactable> mInteractables;
        private Player mPlayer;

        #region Properties

        public List<Interactable> Interactables 
        { 
            get
            {
                return mInteractables;
            }
            set
            {
                mInteractables = value;
            }
        }

        #endregion

        public InteractionManager(Player player)
        {
            this.mPlayer = player;
            this.mInteractables = new List<Interactable>();
        }

        /// <summary>
        /// Updates all Interactables, initiates Interactions.
        /// </summary>
        /// <param name="gameTime"></param>
        public void Update(GameTime gameTime)
        {
            // Update every interactable in mInteractables
            foreach (Interactable interactable in mInteractables)
            {
                interactable.Update(gameTime);
            }

            ///////////////////////////////////////////////////////////////////////////

            // Loop through every interactable the player is touching
            foreach (Interactable interactable in mInteractables)
            {
                // If the player is touching the interactable
                if (interactable.TouchingPlayer && InputManager.HasButtonBeenPressed(Keys.A))
                {
                    bool interactionDone = false;

                    // Loop through each possible interaction in the interactable
                    foreach (KeyValuePair<string, Interaction> interaction in interactable.Interactions)
                    {                         
                        if (!interactionDone)
                        {
                            // If player has item needed for interaction
                            // and interactable is in the right state
                            foreach (Item item in mPlayer.Items)
                            {
                                if (interaction.Value.Item == item &&
                                    interactable.CurrentState == interaction.Value.State)
                                { 
                                    // Set the interactable's delay
                                    interaction.Value.Interactable.InteractDelay = interaction.Value.Delay;

                                    // Set the player's delay
                                    mPlayer.CurrentDelay = interaction.Value.Delay;

                                    // Perform the relevant interaction
                                    interaction.Value.Interactable.PerformInteraction(interaction.Key);                                  

                                    interactionDone = true;
                                }
                            }
                        }
                    }
                }
            }

            ///////////////////////////////////////////////////////////////////////////
        }
    }
}
