﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using PolygonCollisionLibrary;

namespace ProjectMaternity
{
    class Bed : Interactable
    {
        private Patient mPatient;
        protected Random mRandom;

        #region Properties
        public Patient Patient
        {
            get
            {
                return mPatient;
            }
            set
            {
                mPatient = value;
            }
        }
        #endregion

        public Bed(Player player)
            : base (player)
            {
                // Create new Random
                mRandom = new Random();

            // Set the bounding polygon to fit the object
            Vector2[] colVerts = new Vector2[4] {new Vector2(1.0f, 2.0f),
                                                             new Vector2(-1.0f, 2.0f),
                                                             new Vector2(-1.0f, -2.0f),
                                                             new Vector2(1.0f, -2.0f)};
            BoundingPolygon = new Polygon(colVerts, true);

            ////////// Set Up States //////////

            this.mStates = new List<string> { "Empty", "WithPatient", "NeedyPatient", "PatientBeingAssisted", 
                                                "Complicated", "PatientRest", "PatientBaby", "Messy" };
            this.mCurrentState = mStates[0];

            ////////// Set Up Interactions //////////

            this.mInteractions = new Dictionary<string, Interaction>();
            this.mInteractions.Add("GetPatient", new Interaction(this, Item.None, mStates[0], 0.0f));
            this.mInteractions.Add("Clean", new Interaction(this, Item.CleanSheets, mStates[7], 2.0f));
            this.mInteractions.Add("AssistPatient", new Interaction(this, Item.None, mStates[2], 10.0f));
            this.mInteractions.Add("GiveBaby", new Interaction(this, Item.Baby, mStates[6], 0.0f));
        }

        public override void Update(GameTime gameTime)
        {
            // The number of seconds since the last update
            float elapsedTime = (float)gameTime.ElapsedGameTime.TotalMilliseconds / 1000;

            if (mPatient != null)
            {
                if (mPatient.PatientType == PatientType.PregnantLady)
                {
                    switch (mCurrentState)
                    {
                        case "Empty":
                            {
                                if (mInteractDelay > 0)
                                {
                                    mInteractDelay -= elapsedTime;
                                }
                                else
                                {
                                    // Model = cleanMOdel;
                                }
                                break;
                            }
                        case "WithPatient":
                            {
                                if (mInteractDelay > 0)
                                {
                                    mInteractDelay -= elapsedTime;
                                }
                                else
                                {
                                    this.CurrentState = "NeedyPatient";
                                    this.InteractDelay = 15.0f;         // Time the player has until the patient dies
                                }
                                break;
                            }
                        case "NeedyPatient":
                            {
                                if (mInteractDelay > 0)
                                {
                                    mInteractDelay -= elapsedTime;
                                }
                                else
                                {
                                    Patient.CurrentState = "Dead";
                                }
                                break;
                            }
                        case "PatientBeingAssisted":
                            {
                                if (mInteractDelay > 0)
                                {
                                    mInteractDelay -= elapsedTime;
                                }
                                else
                                {
                                    if (mRandom.Next(0, 100) > 70)
                                    {
                                        this.CurrentState = "Complicated";
                                    }
                                    else
                                    {
                                        this.CurrentState = "PatientRest";
                                        this.InteractDelay = 5.0f;
                                    }
                                }
                                break;
                            }

                        case "Complicated":
                            {
                                // Change this so that we have to go to the medicin cabinet
                                this.CurrentState = "PatientRest";
                                break;
                            }

                        case "PatientRest":
                            {
                                if (mInteractDelay > 0)
                                {
                                    mInteractDelay -= elapsedTime;
                                }
                                else
                                {
                                    this.CurrentState = "PatientBaby";
                                }
                                break;
                            }

                        case "Messy":
                            {
                                /*if (model == CleanModel)
                                 * {
                                 *      model = dirtyModel;
                                 * } 
                                 * */
                                break;
                            }
                    }
                }
            }

            base.Update(gameTime);
        }

        public override void PerformInteraction(string interactionName)
        {
            // Select the relevant interaction
            switch (interactionName)
            {
                case "Clean":
                    {
                        mPatient = null;
                        mCurrentState = "Empty";
                        break;
                    }
                case "GetPatient":
                    {
                        // Put patient in bed if following player, and if the bed is empty
                        if (mPlayer.Patient != null && this.Patient == null)
                        {                           
                            mCurrentState = "WithPatient";

                            // Move patient from following player to bed
                            mPatient = mPlayer.Patient;
                            mPatient.CurrentState = "InBed";
                            
                            // Delay until patient will need assistance
                            this.InteractDelay = 5.0f;

                            // Patient is now in bed
                            mPlayer.Patient = null;
                        }
                        break;
                    }
                case "AssistPatient":
                    {
                        if (this.CurrentState == "NeedyPatient")
                        {
                            this.CurrentState = "PatientBeingAssisted";
                        }
                        break;
                    }

                case "GiveBaby":
                    {
                        mPatient.CurrentState = "Leaving";
                        this.CurrentState = "Messy";
                        break;
                    }
            }
        }
    }
}
