﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using PolygonCollisionLibrary;

namespace ProjectMaternity
{
    class Chair : Interactable
    {
        private Patient mPatient;
        protected Random mRandom;

        #region Properties
        public Patient Patient
        {
            get
            {
                return mPatient;
            }
            set
            {
                mPatient = value;
            }
        }
        #endregion

        public Chair(Player player)
            : base (player)
            {
                // Create new Random
                mRandom = new Random();

                Vector2[] colVerts = new Vector2[4] {new Vector2(0.5f, 0.5f),
                                                             new Vector2(-0.5f, 0.5f),
                                                             new Vector2(-0.5f, -0.5f),
                                                             new Vector2(0.5f, -0.5f)};
                BoundingPolygon = new Polygon(colVerts, true);

            ////////// Set Up States //////////

            this.mStates = new List<string> { "Empty", "WithPatient"};
            this.mCurrentState = mStates[0];

            ////////// Set Up Interactions //////////

            this.mInteractions = new Dictionary<string, Interaction>();
            this.mInteractions.Add("GetPatient", new Interaction(this, Item.None, mStates[1], 0.0f));            
        }

        public override void Update(GameTime gameTime)
        {
            // The number of seconds since the last update
            float elapsedTime = (float)gameTime.ElapsedGameTime.TotalMilliseconds / 1000;

            if (mPatient != null)
            {
                if (mPatient.PatientType == PatientType.PregnantLady)
                {
                    switch (mCurrentState)
                    {
                        case "Empty":
                            {
                                if (mInteractDelay > 0)
                                {
                                    mInteractDelay -= elapsedTime;
                                }
                                else
                                {
                                    // Model = cleanMOdel;
                                }
                                break;
                            }
                        case "WithPatient":
                            {
                                if (mInteractDelay > 0)
                                {
                                    mInteractDelay -= elapsedTime;
                                }
                                else
                                {
                                    this.CurrentState = "NeedyPatient";
                                    this.InteractDelay = 15.0f;         // Time the player has until the patient dies
                                }
                                break;
                            }
                        
                    }
                }
            }

            base.Update(gameTime);
        }

        public override void PerformInteraction(string interactionName)
        {
            // Select the relevant interaction
            switch (interactionName)
            {               
                case "GetPatient":
                    {
                        // Put patient in bed if following player, and if the bed is empty
                        if (mPlayer.Patient != null && this.Patient == null)
                        {                           
                            mCurrentState = "WithPatient";

                            // Move patient from following player to bed
                            mPatient = mPlayer.Patient;
                            mPatient.CurrentState = "InBed";
                            
                            // Delay until patient will need assistance
                            this.InteractDelay = 5.0f;

                            // Patient is now in bed
                            mPlayer.Patient = null;
                        }
                        break;
                    }             
            }
        }
    }
}
