﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using PolygonCollisionLibrary;

namespace ProjectMaternity
{
    public enum PatientType
    {
        PregnantLady,
        NotSoPregnantLady
    }

    public class Patient : PathFinder
    {
        // The number of seconds since the last update
        protected float mElapsedTime;

        protected Random mRandom;

        public PatientType PatientType;

        public Patient(List<Polygon> collisionData, Player player, PatientType patientType)
            : base(collisionData, player)
        {
            // Create new Random
            mRandom = new Random();

            // Set Patient Type
            PatientType = patientType;

            ////////// Set Up States //////////

            this.mStates = new List<string> { "Arriving", "Waiting", "Following", "InBed", "Leaving" };
            this.mCurrentState = mStates[0];

            ////////// Set Up Interactions //////////

            this.mInteractions = new Dictionary<string, Interaction>();
            this.mInteractions.Add("GetToFollow", new Interaction(this, Item.None, mStates[1], 0.0f));
            this.mInteractions.Add("GetInBed", new Interaction(this, Item.None, mStates[2], 0.0f ));
        }

        public override void Update(GameTime gameTime)
        {
            // The number of seconds since the last update
            mElapsedTime = (float)gameTime.ElapsedGameTime.TotalMilliseconds / 1000;

            switch (mCurrentState)
            {
                case "Arriving":
                    {
                        // After spawn, move to waiting area
                        break;
                    }

                case "Waiting":
                    {
                        // Wait for a certain amount of time before leaving
                        break;
                    }

                case "Following":
                    {
                        if (mInteractDelay > 0)
                        {
                            mInteractDelay -= mElapsedTime;
                        }
                        else
                        {
                            // Follow the player, use Dom's pathfinding
                            //this.Position = this.mPlayer.Position - new Vector3(5, 0, 0);
                            this.Rotation = this.mPlayer.Rotation;

                            float radAngle = MathHelper.ToRadians(this.mPlayer.Rotation.Y - 90);

                            if (this.mPlayer.Rotation.Y >= 0)
                            {
                                Position = this.mPlayer.Position - (new Vector3(0.5f * (float)Math.Cos(radAngle), 0,
                                    -0.5f * (float)Math.Sin(radAngle))) *6.0f ;
                            }
                            // If the rotation is clockwise
                            else
                            {
                                Position = this.mPlayer.Position - (new Vector3(0.5f * (float)Math.Cos(radAngle), 0,
                                    0.5f * (float)Math.Sin(radAngle))) * 6.0f;
                            }
                        }
                        break;
                    }

                case "Leaving":
                    {
                        // Leave the hospital
                        break;
                    }
            }

            base.Update(gameTime);
        }

        public override void PerformInteraction(string interactionName)
        {
            // Select the relevant interaction
            switch (interactionName)
            {
                case "GetToFollow":
                    {
                        mPlayer.Patient = this;
                        mCurrentState = "Following";
                        break;
                    }
                case "GetInBed":
                    {
                        break;
                    }
            }
        }
    }
}
