﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using TotallyRadEngine10;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using System.IO;

namespace ProjectMaternity
{
    public enum Locations
    {
        Rajasthan = 1,
        Kathmandu = 2
    }

    class MapState : State
    {
        private Locations mCurrentLocation;
        private int mPrevLevel;
        private PersistentData mPersistentData;

        public override void Load(PersistentData persistent)
        {
            // Loads initial location data from level files
            if (!persistent.Booleans["LocationDataLoaded"])
            {
                InitializeLocationData(persistent);
            }

            // Update data related to the level last played
            else if (mPrevLevel != 0)
            {
                UpdateLocationData(persistent);
            }

            // Store local copy of persistent data for access in Update()
            mPersistentData = persistent;

            mCurrentLocation = Locations.Rajasthan;
            mPrevLevel = persistent.Integers["LevelNo"];

            mUI.Add("Map", new MapUI());
        }

        private void InitializeLocationData(PersistentData persistent)
        {
            Array locations = Enum.GetValues(typeof(Locations));

            // Load location data for each level
            for (int i = 1; i < 3; i++)
            {
                StreamReader reader = new StreamReader("Content\\Levels\\" + i + ".level");

                string locationName = locations.GetValue(i - 1).ToString(); 
                string fileLine = reader.ReadLine();

                // Read in initial stats data
                fileLine = reader.ReadLine();
                fileLine = reader.ReadLine();
                fileLine = reader.ReadLine();
                persistent.Integers[locationName + "MaternalDeaths"] = int.Parse(fileLine);
                
                //persistent.Integers[locationName + "BirthComplications"] = int.Parse(fileLine);
                fileLine = reader.ReadLine();
                //persistent.Integers[locationName + "BabyDeaths"] = int.Parse(fileLine);
                fileLine = reader.ReadLine();
                //persistent.Integers[locationName + "MaternalDeaths"] = int.Parse(fileLine);
                fileLine = reader.ReadLine();
                fileLine = reader.ReadLine();
                persistent.Integers[locationName + "Objective"] = int.Parse(fileLine);

                reader.Close();
            }

            // Only load this data once
            persistent.Booleans["LocationDataLoaded"] = true;
        }

        private void UpdateLocationData(PersistentData persistent)
        {
            Array locations = Enum.GetValues(typeof(Locations));
            string locationName = locations.GetValue(mPrevLevel).ToString();

            // Update the relevant location's stats based on play session
            /*persistent.Integers[locationName + "TreatedPatientCount"] = persistent.Integers["PatientsTreated"];
            persistent.Integers[locationName + "BirthComplications"] = persistent.Integers["ComplicationCount"];
            persistent.Integers[locationName + "BabyDeaths"] = persistent.Integers["InfantDeaths"];
            persistent.Integers[locationName + "MaternalDeaths"] = persistent.Integers["MaternalDeaths"];*/
            persistent.Integers[locationName + "MaternalDeaths"] = persistent.Integers["MaternalDeaths"];
            persistent.Integers[locationName + "Objective"] = persistent.Integers["Objective"];
            
        }

        public override void Update(GameTime gameTime)
        {
            UpdateInput(gameTime);

            string locationName = mCurrentLocation.ToString(); // Name of the current location           


            /*mTotalPatients = mPersistentData.Integers[locationName + "BabyDeaths"] + mPersistentData.Integers[locationName + "TreatedPatientCount"];
            mInfantMortalityRate = mPersistentData.Integers[locationName + "BabyDeaths"] * 100.0f / mTotalPatients;
            mMortalityRate = mPersistentData.Integers[locationName + "MaternalDeaths"] * 100.0f / mTotalPatients;*/

            (mUI["Map"] as MapUI).CurrentLocation = mCurrentLocation;
            /*(mUI["Map"] as MapUI).TreatedPatients = mPersistentData.Integers[locationName + "TreatedPatientCount"];
            (mUI["Map"] as MapUI).TotalPatients = mTotalPatients;
            (mUI["Map"] as MapUI).InfantMortalityRate = mInfantMortalityRate;
            (mUI["Map"] as MapUI).MortalityRate = mMortalityRate;*/

            (mUI["Map"] as MapUI).MortalityRate = mPersistentData.Integers[locationName + "MaternalDeaths"];
            (mUI["Map"] as MapUI).Objective = mPersistentData.Integers[locationName + "Objective"];

            // Update the UI
            foreach (KeyValuePair<string, UI> ui in mUI)
            {
                ui.Value.Update();
            }
        }

        private void UpdateInput(GameTime gameTime)
        {
            if (InputManager.HasButtonBeenPressed(Keys.Up) || InputManager.HasButtonBeenPressed(Buttons.DPadUp))
            {
                switch (mCurrentLocation)
                {
                    case Locations.Rajasthan:
                        //mCurrentLocation = Locations.Kathmandu;
                        break;
                    case Locations.Kathmandu:
                        mCurrentLocation = Locations.Rajasthan;
                        break;
                }
            }

            if (InputManager.HasButtonBeenPressed(Keys.Down) || InputManager.HasButtonBeenPressed(Buttons.DPadDown))
            {
                switch (mCurrentLocation)
                {
                    case Locations.Rajasthan:
                        mCurrentLocation = Locations.Kathmandu;
                        break;
                    case Locations.Kathmandu:
                        //mCurrentLocation = Locations.Rajasthan;
                        break;
                }
            } //

            if (InputManager.HasButtonBeenPressed(Keys.Enter) || InputManager.HasButtonBeenPressed(Buttons.A))
            {
                switch (mCurrentLocation)
                {
                    case Locations.Rajasthan:
                        {
                            PlayState nextLevel = new PlayState();
                            nextLevel.LevelNumber = 1;

                            LoadingState loadState = new LoadingState();
                            loadState.StateToLoad = nextLevel;

                            StateManager.SetState(loadState);

                            break;
                        }

                    case Locations.Kathmandu:
                        {
                            PlayState nextLevel = new PlayState();
                            // Change this when the new level is added (2)
                            nextLevel.LevelNumber = 2;

                            LoadingState loadState = new LoadingState();
                            loadState.StateToLoad = nextLevel;

                            StateManager.SetState(loadState);                          

                            break;
                        }
                }
            }
        }

        public override void Draw(Renderer renderer, GameTime gameTime)
        {
            // Draw the UI
            foreach (KeyValuePair<string, UI> ui in mUI)
            {
                ui.Value.Draw(renderer.SpriteBatch);
            }
        }

        public override void Unload(PersistentData persistent)
        {
        }


    }
}
