﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using TotallyRadEngine10;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace ProjectMaternity
{
    public enum MenuStates
    {
        START_GAME,
        CREDITS,
        QUIT,
    }

    class MenuState : State
    {
        MenuStates mCurrentMenuState;  

        public override void Load(PersistentData persistent)
        {
            mCurrentMenuState = MenuStates.START_GAME;

            mUI.Add("Background", new BackgroundUI());
            mUI.Add("MainMenu", new MainMenuUI());
            mUI.Add("Credits", new CreditsUI());

            //mUI["Background"].Visible = false;
            mUI["Credits"].Visible = false;
        }

        public override void Update(Microsoft.Xna.Framework.GameTime gameTime)
        {
            UpdateInput(gameTime);           

            switch (mCurrentMenuState)
            {
                case MenuStates.START_GAME:
                    {
                        mUI["MainMenu"].GetText("Start").Animation = AnimationType.Pulsate;
                        mUI["MainMenu"].GetText("Start").Colour = Color.Red;

                        mUI["MainMenu"].GetText("Credits").Animation = AnimationType.None;
                        mUI["MainMenu"].GetText("Credits").Colour = Color.White;
                        mUI["MainMenu"].GetText("Credits").Scale = 1.0f;

                        mUI["MainMenu"].GetText("Exit").Animation = AnimationType.None;
                        mUI["MainMenu"].GetText("Exit").Colour = Color.White;
                        mUI["MainMenu"].GetText("Exit").Scale = 1.0f;

                        break;
                    }
                case MenuStates.CREDITS:
                    {
                        mUI["MainMenu"].GetText("Start").Animation = AnimationType.None;
                        mUI["MainMenu"].GetText("Start").Colour = Color.White;
                        mUI["MainMenu"].GetText("Start").Scale = 1.0f;

                        mUI["MainMenu"].GetText("Credits").Animation = AnimationType.Pulsate;
                        mUI["MainMenu"].GetText("Credits").Colour = Color.Red;

                        mUI["MainMenu"].GetText("Exit").Animation = AnimationType.None;
                        mUI["MainMenu"].GetText("Exit").Colour = Color.White;
                        mUI["MainMenu"].GetText("Exit").Scale = 1.0f;

                        break;
                    }
                case MenuStates.QUIT:
                    {
                        mUI["MainMenu"].GetText("Start").Animation = AnimationType.None;
                        mUI["MainMenu"].GetText("Start").Colour = Color.White;
                        mUI["MainMenu"].GetText("Start").Scale = 1.0f;

                        mUI["MainMenu"].GetText("Credits").Animation = AnimationType.None;
                        mUI["MainMenu"].GetText("Credits").Colour = Color.White;
                        mUI["MainMenu"].GetText("Credits").Scale = 1.0f;

                        mUI["MainMenu"].GetText("Exit").Animation = AnimationType.Pulsate;
                        mUI["MainMenu"].GetText("Exit").Colour = Color.Red;                     

                        break;
                    }
            }

            // Update the UI
            foreach (KeyValuePair<string, UI> ui in mUI)
            {
                ui.Value.Update();
            }  
        }

        private void UpdateInput(GameTime gameTime)
        {
            if (!mUI["Credits"].Visible)
            {
                if (InputManager.HasButtonBeenPressed(Keys.Up) || InputManager.HasButtonBeenPressed(Buttons.DPadUp))
                {
                    switch (mCurrentMenuState)
                    {
                        case MenuStates.START_GAME:

                            break;
                        case MenuStates.CREDITS:
                            mCurrentMenuState = MenuStates.START_GAME;
                            break;
                        case MenuStates.QUIT:
                            mCurrentMenuState = MenuStates.CREDITS;
                            break;
                    }
                }

                if (InputManager.HasButtonBeenPressed(Keys.Down) || InputManager.HasButtonBeenPressed(Buttons.DPadDown))
                {
                    switch (mCurrentMenuState)
                    {
                        case MenuStates.START_GAME:
                            mCurrentMenuState = MenuStates.CREDITS;
                            break;
                        case MenuStates.CREDITS:
                            mCurrentMenuState = MenuStates.QUIT;
                            break;
                        case MenuStates.QUIT:
                            break;
                    }
                }

                if (InputManager.HasButtonBeenPressed(Keys.Enter) || InputManager.HasButtonBeenPressed(Buttons.A))
                {
                    switch (mCurrentMenuState)
                    {
                        case MenuStates.START_GAME:
                            StateManager.SetState(new ControlScreenState());
                            break;
                        case MenuStates.CREDITS:
                            {
                                mUI["Credits"].Visible = true;
                                mUI["MainMenu"].Visible = false;
                                break;
                            }
                        case MenuStates.QUIT:
                            mActive = false;
                            break;
                    }
                }
            }
            else
            {
                if (InputManager.HasButtonBeenPressed(Keys.Enter) ||
                InputManager.HasButtonBeenPressed(Keys.Space) || InputManager.HasButtonBeenPressed(Buttons.A))
                {
                    mUI["Credits"].Visible = false;
                    mUI["MainMenu"].Visible = true;
                }
            }
        }

        public override void Draw(Renderer renderer, GameTime gameTime)
        { 
            // Draw the UI
            foreach (KeyValuePair<string, UI> ui in mUI)
            {
                ui.Value.Draw(renderer.SpriteBatch);
            }
        }

        public override void Unload(PersistentData persistent)
        {
            persistent.Booleans["LocationDataLoaded"] = false;
        }

     
    }
}
