﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using TotallyRadEngine10;
using Microsoft.Xna.Framework;
using PolygonCollisionLibrary;
using Microsoft.Xna.Framework.Graphics;

namespace ProjectMaternity
{
    public class MicroPathFinder
    {
        static List<EnvironmentNode> mEnvironmentMap;
        //List<PathNode> mNodeNetwork;
        Entity mEntity;
        List<PathNode> mPath;
        PathNode mMoveToNode;
        int mPathIndex;
        bool mFollowingPath;
        bool mPathFollowed;
        bool mFindingPath;
        public bool AutoFollowPath = false;

        // Speed at which Entitys follow their path
        private float mFollowSpeed = 0.2f;

        Vector2 mStart = Vector2.Zero;
        Vector2 mTarget = Vector2.Zero;
        List<PathNode> mOpenList;
        List<PathNode> mClosedList;

        // Increase this value to reduce path computation time
        // Will also make path less accurate and could break the system at much larger values
        public static float mNodeSize = 1.0f;
        static int mStartX = -50;
        static int mEndX = 45;
        static int mStartZ = -50;
        static int mEndZ = 34;
        static int mNodeCountX = (int)((mEndX - mStartX) / mNodeSize);
        static int mNodeCountY = (int)((mEndZ - mStartZ) / mNodeSize);

        #region Properties!

        public PathNode MoveToNode
        {
            get
            {
                return mMoveToNode;
            }
        }

        public bool FindingPath
        {
            get
            {
                return mFindingPath;
            }
        }

        public bool HasPathBeenFound
        {
            get
            {
                return mFindingPath == false && mPath.Count > 0;
            }
        }

        public bool IsFollowingPath
        {
            get
            {
                return mFollowingPath;
            }
        }

        public bool PathFollowed
        {
            get
            {
                return mPathFollowed;
            }
        }

        public Vector3 TargetPosition3D
        {
            get { return new Vector3(mTarget.X, 0, mTarget.Y); }
        }
        public Vector2 TargetPosition2D
        {
            get { return mTarget; }
        }

        public Vector3 StartPosition3D
        {
            get { return new Vector3(mStart.X, 0, mStart.Y); }
        }
        public Vector2 StartPosition2D
        {
            get { return mStart; }
        }

        #endregion

        public static void CreateEnvironmentMap(List<Polygon> collisionData)
        {
            mEnvironmentMap = new List<EnvironmentNode>();

            // Determine which nodes are blocked based upon given collision data
            for (int x = 0; x < mNodeCountX; x++)
            {
                for (int y = 0; y < mNodeCountY; y++)
                {
                    EnvironmentNode newNode = new EnvironmentNode();
                    newNode.Position.X = mStartX + (x * mNodeSize);
                    newNode.Position.Y = mStartZ + (y * mNodeSize);

                    float squareSize = 0.5f * mNodeSize;
                    Vector2[] colVerts = new Vector2[4] {new Vector2(squareSize, squareSize),
                                                             new Vector2(-squareSize, squareSize),
                                                             new Vector2(-squareSize, -squareSize),
                                                             new Vector2(squareSize, -squareSize)};

                    newNode.BoundingPolygon = new Polygon(colVerts, true);
                    newNode.BoundingPolygon.Update(newNode.Position, 0, 1);

                    for (int i = 0; i < collisionData.Count; i++)
                    {
                        Polygon colPoly = collisionData[i];
                        if (newNode.BoundingPolygon.Intersects(ref colPoly))
                        {
                            newNode.Blocked = true;
                        }
                    }

                    mEnvironmentMap.Add(newNode);
                }
            }
        }

        static PathNode CreateNode(Vector2 coords)
        {
            PathNode newNode = new PathNode();
            newNode.Position = new Vector2(coords.X, coords.Y);

            float squareSize = 0.5f * mNodeSize;
            Vector2[] colVerts = new Vector2[4] {new Vector2(squareSize, squareSize),
                                                             new Vector2(-squareSize, squareSize),
                                                             new Vector2(-squareSize, -squareSize),
                                                             new Vector2(squareSize, -squareSize)};

            newNode.BoundingPolygon = new Polygon(colVerts, true);
            newNode.BoundingPolygon.Update(newNode.Position, 0, 1);

            //newNode.BoundingPolygon = new Polygon(newNode.Position, 4, mNodeSize / 2.0f, true);

            return newNode;
        }

        public MicroPathFinder(Entity entity) // : base(player)
        {
            mEntity = entity;
            mPath = new List<PathNode>();
            mMoveToNode = null;
            mFollowingPath = false;
            mPathFollowed = false;
            mFindingPath = false;
            mOpenList = new List<PathNode>();
            mClosedList = new List<PathNode>();
            //mCollisionData = collisionData;
        }

        public void CancelCurrentPath()
        {
            mFindingPath = false;
            mFollowingPath = false;
            mPathFollowed = false;
            mOpenList.Clear();
            mClosedList.Clear();
            mPath.Clear();
            mMoveToNode = null;
        }

        public static Vector2 SnapCoords(Vector2 coords)
        {

            float transX = mNodeSize * (float)Math.Ceiling(coords.X / mNodeSize);
            float transY = mNodeSize * (float)Math.Ceiling(coords.Y / mNodeSize);


            //transX -= mNodeSize - (float)(Math.Floor(mNodeSize));
            //transY -= (mNodeSize - (float)(Math.Floor(mNodeSize))) / 2;

            //return new Vector2(transX, transY);
            return coords;
        }

        public static EnvironmentNode GetCollisionMapNode(Vector2 coords)
        {
            EnvironmentNode theNode = new EnvironmentNode();
            theNode.Blocked = true;
            theNode.Position = Vector2.Zero;

            foreach (EnvironmentNode node in mEnvironmentMap)
            {
                /*if (node.Position.X == coords.X && node.Position.Y == coords.Y)
                {
                    theNode = node;
                }*/

                if (coords.X >= (node.Position.X - (mNodeSize / 2)) && coords.X <= (node.Position.X - (mNodeSize / 2)) + mNodeSize &&
                    coords.Y >= (node.Position.Y - (mNodeSize / 2)) && coords.Y <= (node.Position.Y - (mNodeSize / 2)) + mNodeSize)
                {
                    theNode = node;
                }
            }

            return theNode;
        }

        /*public PathNode GetNode(int x, int y)
        {
            PathNode theNode = null;

            foreach (PathNode node in mNodeNetwork)
            {
                if (node.Position.X == x && node.Position.Y == y)
                {
                    theNode = node;
                }
            }

            return theNode;
        }*/

        public bool FindPathTo(Vector3 target)
        {
            if (!mFindingPath)
            {
                //mNodeNetwork = new List<PathNode>(mCollisionMap);
                mFindingPath = true;
                mOpenList.Clear();
                mClosedList.Clear();
                mPath.Clear();

                //mStart = SnapCoords(new Vector2((int)Math.Floor(mEntity.Position.X), (int)Math.Floor(mEntity.Position.Z)));
                mStart = new Vector2(mEntity.Position.X, mEntity.Position.Z);

                // Can't pathfind if starting from a blocked node
                EnvironmentNode startEnvNode = GetCollisionMapNode(mStart);
                mStart = startEnvNode.Position;
                //if (!startEnvNode.Blocked)
                {
                    PathNode startNode = CreateNode(mStart);
                    mOpenList.Add(startNode);

                    mTarget = new Vector2(target.X, target.Z);

                    EnvironmentNode targetNode = GetCollisionMapNode(mTarget);
                    mTarget = targetNode.Position;

                    // Can't pathfind to a blocked node
                    //if (!targetNode.Blocked)
                    {
                        PathNode newNode = CreateNode(mTarget);
                        newNode.G = 9999;
                        newNode.H = 0;
                        newNode.F = 9999;
                        mOpenList.Add(newNode);
                    }
                    /*else
                    {
                        mFindingPath = false;
                    }*/
                }
                /*else
                {
                    mFindingPath = false;
                }*/

                //ProcessNode(start);
            }

            return mFindingPath;
        }

        public bool FindPathFromTo(Vector3 start, Vector3 target)
        {
            if (!mFindingPath)
            {
                //mNodeNetwork = new List<PathNode>(mCollisionMap);
                mFindingPath = true;
                mOpenList.Clear();
                mClosedList.Clear();
                mPath.Clear();

                //mStart = SnapCoords(new Vector2((int)Math.Floor(mEntity.Position.X), (int)Math.Floor(mEntity.Position.Z)));
                mStart = new Vector2(start.X, start.Z);//new Vector2(mEntity.Position.X, mEntity.Position.Z);

                // Can't pathfind if starting from a blocked node
                EnvironmentNode startEnvNode = GetCollisionMapNode(mStart);
                mStart = startEnvNode.Position;
                // Can't pathfind to a blocked node
                // so sample the nodes around the target to try to find one that isn't blocked
                Vector2 offset = new Vector2(0, 0);
                while (startEnvNode.Blocked)
                {
                    if (offset == new Vector2(0, 0))
                    {
                        offset = new Vector2(1, 0);
                    }
                    else if (offset == new Vector2(1, 0))
                    {
                        offset = new Vector2(-1, 0);
                    }
                    else if (offset == new Vector2(-1, 0))
                    {
                        offset = new Vector2(0, 1);
                    }
                    else if (offset == new Vector2(0, 1))
                    {
                        offset = new Vector2(0, -1);
                    }
                    else if (offset == new Vector2(0, -1))
                    {
                        offset = new Vector2(1, 1);
                    }
                    else if (offset == new Vector2(1, 1))
                    {
                        offset = new Vector2(-1, -1);
                    }
                    else if (offset == new Vector2(-1, -1))
                    {
                        offset = new Vector2(-1, 1);
                    }
                    else if (offset == new Vector2(-1, 1))
                    {
                        offset = new Vector2(1, -1);
                    }
                    else if (offset == new Vector2(1, -1))
                    {
                        offset = new Vector2(2, 0);
                    }
                    else if (offset == new Vector2(2, 0))
                    {
                        offset = new Vector2(-2, 0);
                    }
                    else if (offset == new Vector2(-2, 0))
                    {
                        offset = new Vector2(0, 2);
                    }
                    else if (offset == new Vector2(0, 2))
                    {
                        offset = new Vector2(0, -2);
                    }
                    else if (offset == new Vector2(0, -2))
                    {
                        offset = new Vector2(2, 2);
                    }
                    else if (offset == new Vector2(2, 2))
                    {
                        offset = new Vector2(-2, -2);
                    }
                    else if (offset == new Vector2(-2, -2))
                    {
                        offset = new Vector2(-2, 2);
                    }
                    else if (offset == new Vector2(-2, 2))
                    {
                        offset = new Vector2(2, -2);
                    }

                    startEnvNode = GetCollisionMapNode(mStart + offset);
                    mStart = startEnvNode.Position;
                }

                //if (!startEnvNode.Blocked)
                {
                    PathNode startNode = CreateNode(mStart);
                    mOpenList.Add(startNode);

                    mTarget = new Vector2(target.X, target.Z);

                    EnvironmentNode targetNode = GetCollisionMapNode(mTarget);
                    mTarget = targetNode.Position;

                    // Can't pathfind to a blocked node
                    // so sample the nodes around the target to try to find one that isn't blocked
                    offset = new Vector2(0, 0);
                    while (targetNode.Blocked)
                    {
                        if (offset == new Vector2(0, 0))
                        {
                            offset = new Vector2(1, 0);
                        }
                        else if (offset == new Vector2(1, 0))
                        {
                            offset = new Vector2(-1, 0);
                        }
                        else if (offset == new Vector2(-1, 0))
                        {
                            offset = new Vector2(0, 1);
                        }
                        else if (offset == new Vector2(0, 1))
                        {
                            offset = new Vector2(0, -1);
                        }
                        else if (offset == new Vector2(0, -1))
                        {
                            offset = new Vector2(1, 1);
                        }
                        else if (offset == new Vector2(1, 1))
                        {
                            offset = new Vector2(-1, -1);
                        }
                        else if (offset == new Vector2(-1, -1))
                        {
                            offset = new Vector2(-1, 1);
                        }
                        else if (offset == new Vector2(-1, 1))
                        {
                            offset = new Vector2(1, -1);
                        }
                        else if (offset == new Vector2(1, -1))
                        {
                            offset = new Vector2(2, 0);
                        }
                        else if (offset == new Vector2(2, 0))
                        {
                            offset = new Vector2(-2, 0);
                        }
                        else if (offset == new Vector2(-2, 0))
                        {
                            offset = new Vector2(0, 2);
                        }
                        else if (offset == new Vector2(0, 2))
                        {
                            offset = new Vector2(0, -2);
                        }
                        else if (offset == new Vector2(0, -2))
                        {
                            offset = new Vector2(2, 2);
                        }
                        else if (offset == new Vector2(2, 2))
                        {
                            offset = new Vector2(-2, -2);
                        }
                        else if (offset == new Vector2(-2, -2))
                        {
                            offset = new Vector2(-2, 2);
                        }
                        else if (offset == new Vector2(-2, 2))
                        {
                            offset = new Vector2(2, -2);
                        }

                        targetNode = GetCollisionMapNode(mTarget + offset);
                        mTarget = targetNode.Position;
                    }


                    //if (!)
                    {
                        PathNode newNode = CreateNode(mTarget);
                        newNode.G = 9999;
                        newNode.H = 0;
                        newNode.F = 9999;
                        mOpenList.Add(newNode);
                    }
                    /*else
                    {
                        mFindingPath = false;
                    }*/
                }
                /*else
                {
                    mFindingPath = false;
                }*/

                //ProcessNode(start);
            }

            return mFindingPath;
        }

        private bool IsNodeOnClosedList(Vector2 coords)
        {
            bool nodeFound = false;

            foreach (PathNode aNode in mClosedList)
            {
                if (coords.X == aNode.Position.X && coords.Y == aNode.Position.Y)
                {
                    nodeFound = true;
                }
            }

            return nodeFound;
        }

        private bool IsNodeOnOpenList(Vector2 coords)
        {
            bool nodeFound = false;

            foreach (PathNode aNode in mOpenList)
            {
                if (coords.X == aNode.Position.X && coords.Y == aNode.Position.Y)
                {
                    nodeFound = true;
                }
            }

            return nodeFound;
        }

        private PathNode FindExistingNodeOnOpenList(Vector2 coords)
        {
            PathNode exisitingNode = new PathNode();

            foreach (PathNode aNode in mOpenList)
            {
                if (coords.X == aNode.Position.X && coords.Y == aNode.Position.Y)
                {
                    exisitingNode = aNode;
                }
            }

            return exisitingNode;
        }

        private PathNode FindExistingNodeOnClosedList(Vector2 coords)
        {
            PathNode exisitingNode = new PathNode();

            foreach (PathNode aNode in mClosedList)
            {
                if (coords.X == aNode.Position.X && coords.Y == aNode.Position.Y)
                {
                    exisitingNode = aNode;
                }
            }

            return exisitingNode;
        }

        private void ProcessAdjacentNode(PathNode originNode, Vector2 coords)
        {
            // Find all valid adjacent nodes
            Vector2 adjNodePos = originNode.Position + coords;

            if (adjNodePos.X > mStartX && adjNodePos.X < mEndX &&
                adjNodePos.Y > mStartZ && adjNodePos.Y < mEndZ)
            {
                PathNode adjNode = CreateNode(SnapCoords(adjNodePos));

                if (!GetCollisionMapNode(SnapCoords(adjNodePos)).Blocked || adjNodePos == mTarget)
                {
                    if (!IsNodeOnClosedList(adjNodePos) && !IsNodeOnOpenList(adjNodePos))
                    {
                        adjNode.Parent = originNode;
                        adjNode.G = adjNode.Parent.G + ((coords.X != 0 && coords.Y != 0) ? 14 : 10);
                        adjNode.H = (int)Math.Floor(Vector2.Distance(originNode.Position + coords, mTarget)) * 10;
                        adjNode.F = adjNode.G + adjNode.H;
                        mOpenList.Add(adjNode);
                    }
                    else if (!IsNodeOnClosedList(adjNodePos) && IsNodeOnOpenList(adjNodePos))
                    {
                        PathNode existingNode = FindExistingNodeOnOpenList(adjNodePos);

                        int newG = originNode.G + ((coords.X != 0 && coords.Y != 0) ? 14 : 10);

                        if (newG < existingNode.G)
                        {
                            existingNode.Parent = originNode;
                            existingNode.G = newG;
                            existingNode.F = existingNode.G + existingNode.H;
                        }
                    }
                }
            }
        }

        private void ProcessNode(PathNode node)
        {
            // Negative Z node
            ProcessAdjacentNode(node, new Vector2(0, -mNodeSize));
            // Positive Z node
            ProcessAdjacentNode(node, new Vector2(0, mNodeSize));
            // Negative X node
            ProcessAdjacentNode(node, new Vector2(-mNodeSize, 0));
            // Positive X node
            ProcessAdjacentNode(node, new Vector2(mNodeSize, 0));
            // Negative Z, negative X node
            ProcessAdjacentNode(node, new Vector2(-mNodeSize, -mNodeSize));
            // Negative Z, positive X node
            ProcessAdjacentNode(node, new Vector2(mNodeSize, -mNodeSize));
            // Positive Z, negative X node
            ProcessAdjacentNode(node, new Vector2(-mNodeSize, mNodeSize));
            // Positive Z, positive X node
            ProcessAdjacentNode(node, new Vector2(mNodeSize, mNodeSize));
        }

        public void FollowPath()
        {
            if (!mFindingPath && !mFollowingPath && !mPathFollowed)
            {
                mFollowingPath = true;
            }
        }

        public void Update(GameTime gameTime)
        {
            if (mFindingPath)
            {
                int lowestFScore = 9999;
                int nextNodeIndex = -1;

                // Find the next node to process
                for (int i = 0; i < mOpenList.Count; i++)
                {
                    PathNode node = mOpenList[i];

                    if (node.F < lowestFScore)
                    {
                        lowestFScore = node.F;
                        nextNodeIndex = i;
                    }
                }

                if (nextNodeIndex != -1)
                {
                    // Remove this node from the open list since we have already processed all of it's adjacent nodes
                    PathNode nextNode = mOpenList[nextNodeIndex];
                    mClosedList.Add(nextNode);
                    mOpenList.RemoveAt(nextNodeIndex);

                    ProcessNode(nextNode);

                    if (IsNodeOnClosedList(mTarget))
                    {
                        // Save path
                        PathNode currentNode = FindExistingNodeOnClosedList(mTarget);                

                        // If the currentNode is null it means that the path (including the target)
                        // is just two nodes long and chances are that the entity is already on the last node
                        // so by forcing that node to be added to the path, the path will be instantly
                        // flagged as found and followed
                        if (currentNode != null && currentNode.Parent != null)
                        {
                            while (currentNode.Parent != null)
                            {
                                mPath.Add(currentNode);
                                currentNode = currentNode.Parent;
                            }
                        }
                        else
                        {
                            mPath.Add(currentNode);
                        }

                        mFindingPath = false;

                        if (AutoFollowPath)
                        {
                            FollowPath();
                        }
                    }
                    else if (mOpenList.Count == 0)
                    {
                        // No path found
                        mFindingPath = false;
                    }
                }
                else
                {
                    // No path found
                    mFindingPath = false;
                }
            }
            else if (mFollowingPath)
            {
                if (mMoveToNode == null)
                {
                    if (mPath.Count > 0)
                    {
                        mPathIndex = mPath.Count - 1;
                        mMoveToNode = mPath[mPathIndex];
                    }
                    else
                    {
                        mFollowingPath = false;
                        mPathFollowed = true;
                    }
                }
                else
                {
                    if (mEntity.Position.X < mMoveToNode.Position.X - 0.4f)
                    {
                        mEntity.Position.X += mFollowSpeed * (float)Math.Ceiling(gameTime.ElapsedGameTime.Milliseconds / ((1.0f / 60.0f) * 1000.0f));
                    }

                    if (mEntity.Position.X > mMoveToNode.Position.X + 0.4f)
                    {
                        mEntity.Position.X -= mFollowSpeed * (float)Math.Ceiling(gameTime.ElapsedGameTime.Milliseconds / ((1.0f / 60.0f) * 1000.0f));
                    }

                    if (mEntity.Position.Z < mMoveToNode.Position.Y - 0.4f)
                    {
                        mEntity.Position.Z += mFollowSpeed * (float)Math.Ceiling(gameTime.ElapsedGameTime.Milliseconds / ((1.0f / 60.0f) * 1000.0f));
                    }

                    if (mEntity.Position.Z > mMoveToNode.Position.Y + 0.4f)
                    {
                        mEntity.Position.Z -= mFollowSpeed * (float)Math.Ceiling(gameTime.ElapsedGameTime.Milliseconds / ((1.0f / 60.0f) * 1000.0f));
                    }

                    if (mEntity.Position.X >= mMoveToNode.Position.X - 0.4f && mEntity.Position.X <= mMoveToNode.Position.X + 0.4f &&
                        mEntity.Position.Z >= mMoveToNode.Position.Y - 0.4f && mEntity.Position.Z <= mMoveToNode.Position.Y + 0.4f)
                    {
                        if (mPathIndex > 0)
                        {
                            mPathIndex--;
                            mMoveToNode = mPath[mPathIndex];

                            // Vector between the entity and the next node
                            Vector2 direction = Vector2.Normalize(mMoveToNode.Position - new Vector2(mEntity.Position.X, mEntity.Position.Z));

                            // Make entity face in direction of the next node
                            mEntity.Rotation.Y = MathHelper.ToDegrees((float)Math.Atan2(direction.X, direction.Y));
                        }
                        else
                        {
                            mFollowingPath = false;
                            mPathFollowed = true;
                        }
                    }
                }
            }

            //base.Update(gameTime);
        }

        public void DrawPath(Renderer renderer)
        {
            if (mFindingPath || mFollowingPath || HasPathBeenFound)
            {
                foreach (PathNode node in mPath)
                {
                    node.BoundingPolygon.RenderDataColor = Color.LightGreen;
                    renderer.AddPolygonToDraw(node.BoundingPolygon);
                }

                foreach (PathNode node in mOpenList)
                {
                    node.BoundingPolygon.RenderDataColor = Color.Blue;
                    renderer.AddPolygonToDraw(node.BoundingPolygon);
                }

                foreach (PathNode node in mClosedList)
                {
                    node.BoundingPolygon.RenderDataColor = Color.Orange;
                    renderer.AddPolygonToDraw(node.BoundingPolygon);
                }

            }

            GetCollisionMapNode(mStart).BoundingPolygon.RenderDataColor = Color.Black;
            renderer.AddPolygonToDraw(GetCollisionMapNode(mStart).BoundingPolygon);

            GetCollisionMapNode(mTarget).BoundingPolygon.RenderDataColor = Color.Black;
            renderer.AddPolygonToDraw(GetCollisionMapNode(mTarget).BoundingPolygon);
        }

        public static void DrawEnvironmentMap(Renderer renderer)
        {
            foreach (EnvironmentNode node in mEnvironmentMap)
            {
                if (node.Blocked)
                {
                    node.BoundingPolygon.RenderDataColor = Color.OrangeRed;
                }
                else
                {
                    node.BoundingPolygon.RenderDataColor = Color.LightGreen;
                }

                renderer.AddPolygonToDraw(node.BoundingPolygon);
            }
        }

    }
}
