﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using PolygonCollisionLibrary;
using TotallyRadEngine10;
using Microsoft.Xna.Framework.Graphics;

namespace ProjectMaternity
{
    public enum Item
    {
        None,
        Medical,
        CleanSheets,
        DirtySheets
    }

    public class Nurse : Interactable
    {
        public Polygon InteractionPoly;
        protected Item mItem;
        public Renderable ItemRenderable;

        public Item CurrentItem
        {
            get
            {
                return mItem;
            }
            set
            {
                mItem = value;
            }
        }

        public Nurse() : base()
        {
            InteractionPoly = new Polygon(new Vector2(Position.X, Position.Z), 15, 3.5f, true);

            // Create Items list, set default values
            CurrentItem = Item.None;

            ItemRenderable = new Renderable();
            ItemRenderable.Position = Position;
            ItemRenderable.Rotation = Rotation;
            ItemRenderable.Scale = new Vector3(0.425f, 0.425f, 0.425f);
            ItemRenderable.Model = ContentCache.GetModel("Models\\Animated\\Sheets");
            ItemRenderable.Colour = Color.White;
        }

        protected override bool OnInteraction(Interactable other)
        {
            bool success = true;
            return success;
        }

        public override void Continue(GameTime gameTime)
        {
            foreach (Interaction interaction in mInteractions)
            {
                if (interaction.Other is Bed)
                {
                    // Vector between the nurse and the bed
                    Vector3 direction = Vector3.Normalize(interaction.Other.Position - this.Position);

                    // Make nurse face the bed
                    this.Rotation.Y = MathHelper.ToDegrees((float)Math.Atan2(direction.X, direction.Z));
                }
            }

            base.Continue(gameTime);
        }

        public override void InteractionComplete(Interaction interaction)
        {
            if (interaction.Other is Bed)
            {
                if (IsAnimationPlaying && !CurrentAnimationName.Contains("Idle"))
                {
                    if (CurrentItem != Item.CleanSheets && CurrentItem != Item.DirtySheets)
                    {
                        SetAnimation("Idle", 0.0f, 1.0f);
                    }
                    else
                    {
                        SetAnimation("IdleSheets", 0.0f, 1.0f);
                    }
                }
            }
        }

        public override void Update(GameTime gameTime)
        {
            // Update the Interaction Polygon
            InteractionPoly.Update(new Vector2(Position.X, Position.Z), MathHelper.ToRadians(-Rotation.Y), 1.0f);

            Continue(gameTime);

            ItemRenderable.Position = Position;
            ItemRenderable.Rotation = Rotation;

            if (mItem == Item.CleanSheets)
            {
                ItemRenderable.Model = ContentCache.GetModel("Models\\Animated\\Sheets");
            }
            else if (mItem == Item.DirtySheets)
            {
                ItemRenderable.Model = ContentCache.GetModel("Models\\Animated\\DirtySheets");
            }
            else if (mItem == Item.Medical)
            {
                ItemRenderable.Model = ContentCache.GetModel("Models\\Animated\\ToolsTray");
            }

            base.Update(gameTime);
        }
    }
}
