﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using TotallyRadEngine10;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework;
using PolygonCollisionLibrary;
using Microsoft.Xna.Framework.Graphics;

namespace ProjectMaternity
{
    public class Patient : Interactable
    {
        public Vector3 SpawnPos;
        bool mGoingBackToSpawnPoint = false;
        List<Chair> mChairs;
        int mChair = -1; // The chair the patient wants to sit in

        public MacroPathFinder MacroPathFinder;
        public MicroPathFinder MicroPathFinder;
        private string mCurrentState;
        private float mUrgencyTimer = 0; // red(20)/orange(40)/green(60)
        float mLowUrgencyBound = 20;
        float mMedUrgencyBound = 40;
        float mHighUrgencyBound = 60;
        //private float mDelayTimer = -1;
        private Random mRandom;
        float mDelay;

        private Color mUrgencyColor = Color.White;
        private bool mSuccessfullyTreated = false;

        public bool HasLeft = false;
        public bool IsAlive = true;
        bool mIsDying = false;

        public int UniqueID;

        Vector3 mLastPos;

        #region Properties

        public string CurrentState
        {
            get
            {
                return mCurrentState;
            }
            set
            {
                mCurrentState = value;
            }
        }

        public float UrgencyTimer
        {
            get
            {
                return mUrgencyTimer;
            }
            set
            {
                mUrgencyTimer = value;
            }
        }

        public float Delay
        {
            get
            {
                return mDelay;
            }
            set
            {
                mDelay = value;
            }
        }

        public Color UrgencyColor
        {
            get
            {
                return mUrgencyColor;
            }
            set
            {
                mUrgencyColor = value;
            }
        }

        public bool SuccesfullyTreated
        {
            get
            {
                return mSuccessfullyTreated;
            }
        }

        #endregion

        // Constructor
        public void SetChairs(ref List<Chair> chairs)
        {
            // Give patient access to all chairs in the level
            mChairs = chairs;

            // Set default patient state to Waiting
            mCurrentState = "Waiting";
        }

        public Patient()
            : base()
        {
            MacroPathFinder = new MacroPathFinder(this);
            MicroPathFinder = new MicroPathFinder(this);

            mRandom = new Random();
            mUrgencyTimer = float.MaxValue;
            SetUrgencyColor();

            Model = ContentCache.GetModel("Models\\Animated\\LadyIdle1");
            InitialiseAnimation("Content\\Models\\Animated\\PatientAnims.tra");
            SetAnimation("Idle", 0.0f, 1.4f);
            Scale = new Vector3(0.85f, 0.85f, 0.85f);
        }

        protected override bool OnInteraction(Interactable other)
        {
            bool success = true;

            if (other is Nurse)
            {
                if (!other.IsInteractingWith<Patient>())
                {
                    if (mCurrentState == "Waiting" && !IsInteractingWith<Bed>())
                    {
                        // Stop sitting / following AI nurse
                        StopInteractionWith<Chair>();
                        StopInteractionWith<Nurse>();

                        // Can no longer be targeted by AI Nurses
                        TargetedBy = null;

                        MacroPathFinder.CancelCurrentPath();
                        BeginInteraction(other, -1);
                    }
                }
            }

            if (other is Bed)
            {
                mDelay = 5; // Seconds
            }

            return success;
        }

        public override void Continue(GameTime gameTime)
        {
            foreach (Interaction interaction in mInteractions)
            {
                if (interaction.Other is Nurse)
                {
                    if (mCurrentState == "Waiting" && !IsInteractingWith<Bed>())
                    {
                        // Vector between the nurse and the patient
                        Vector3 direction = Vector3.Normalize(interaction.Other.Position - this.Position);

                        mLastPos = Position;
                        // Get patient to follow player
                        this.Position = interaction.Other.Position - (direction * 2.5f);

                        // Make patient face in direction of the player
                        this.Rotation.Y = MathHelper.ToDegrees((float)Math.Atan2(direction.X, direction.Z));

                        if (mLastPos != Position)
                        {
                            if (CurrentAnimationName != "Walk")
                            {
                                SetAnimation("Walk", 0.0f, 1.2f);
                            }
                        }
                        else
                        {
                            if (CurrentAnimationName != "Idle")
                            {
                                SetAnimation("Idle", 0.0f, 1.4f);
                            }
                        }
                    }
                }
            }

            base.Continue(gameTime);
        }

        public override void InteractionComplete(Interaction interaction)
        {
            if (interaction.Other is Nurse)
            {
                // We were following the player but now we're not so find a chair
                if (mCurrentState == "Waiting")
                {
                    MacroPathFinder.CancelCurrentPath();
                    mChair = -1;
                    PickAChair();
                }
            }

            if (interaction.Other is Chair)
            {
                //Model = ContentCache.GetModel("Models\\Animated\\PregnantLady");
                SetAnimation("Idle", 0.0f, 1.4f);
            }
        }

        public override void Update(GameTime gameTime)
        {
            base.Update(gameTime);

            // Increase urgency
            if (mUrgencyTimer != -1)
            {
                if (mCurrentState != "Complicated")
                {
                    mUrgencyTimer += (float)gameTime.ElapsedGameTime.TotalMilliseconds / 1000;
                }
                // Complicated patients degrade at twice the rate
                else
                {
                    mUrgencyTimer += (float)(gameTime.ElapsedGameTime.TotalMilliseconds / 1000) * 2;
                }
            }
            else
            {
                mDelay -= (float)gameTime.ElapsedGameTime.TotalMilliseconds / 1000;

                if (mDelay <= 0)
                {
                    if (mCurrentState == "Waiting")
                    {
                        mUrgencyTimer = 0;
                        mCurrentState = "Delivering";
                        AudioPlayer.PlaySound("WomanAssistance");
                    }
                    else if (mCurrentState == "Resting")
                    {
                        // Leave
                        mSuccessfullyTreated = true;
                        mCurrentState = "Leaving";

                        //Model = ContentCache.GetModel("Models\\Animated\\Lady");
                        SetAnimation("Idle", 0.0f, 1.4f);
                        this.Rotation.Y = 0;
                        (this.GetInteractable<Bed>() as Bed).CurrentState = "Dirty";
                        (this.GetInteractable<Bed>() as Bed).Colour = Color.Red;
                        this.StopInteractionWith<Bed>();

                        if (!mGoingBackToSpawnPoint)
                        {
                            // GO BACK FROM WHENCE YOU CAME!
                            mGoingBackToSpawnPoint = true;
                            MacroPathFinder.CancelCurrentPath();
                            MacroPathFinder.FindPathTo(SpawnPos);
                        }
                    }
                }
            }

            SetUrgencyColor();

            // If the urgency exceeds the high urgency bound then the patient's baby has died
            // The patient themselves may die also but this only happens if they are already in bed
            // Maybe there will be a random chance of the patient dying out of bed too
            if (mUrgencyTimer >= mHighUrgencyBound)
            {
                if (IsInteractingWith<Bed>())
                {
                    mIsDying = true;
                }
                else
                {
                    if (mCurrentState != "Leaving")
                    {
                        // Set model colour to white, just incase the player was hovering around them as they decided to leave
                        Colour = Color.White;

                        // Either sitting, walking around or following player
                        StopInteractionWith<Chair>();
                        StopInteractionWith<Nurse>();

                        mCurrentState = "Leaving";

                        if (!mGoingBackToSpawnPoint)
                        {
                            // GO BACK FROM WHENCE YOU CAME!
                            mGoingBackToSpawnPoint = true;
                            MacroPathFinder.CancelCurrentPath();
                            MicroPathFinder.CancelCurrentPath();
                            MacroPathFinder.FindPathTo(SpawnPos);
                        }

                        // Decide whether the patient dies or if their baby dies due to neglect
                        /*Random random = new Random();

                        if (random.Next(0, 100) > 100 - (PlayState.ComplicationChance))
                        {
                            mIsDying = true;
                        }*/
                        mIsDying = true;
                    }
                }
            }

            // The patient's baby has died and the patient has left the hospital
            if (mCurrentState == "Leaving" && MacroPathFinder.HasPathBeenFound && MacroPathFinder.PathFollowed && !mIsDying)
            {
                HasLeft = true;
            }

            if (mIsDying)
            {
                // Patient on bed just fades out of existence
                if (Colour.A > 0)
                {
                    if (Colour.A - 3 > 0)
                    {
                        Colour.A -= 3;
                    }
                    else
                    {
                        Colour.A = 0;
                    }
                }
                else
                {
                    Colour.A = 0;
                    IsAlive = false;

                    if (IsInteractingWith<Bed>())
                    {
                        // Dead people make the bed dirty
                        (this.GetInteractable<Bed>() as Bed).CurrentState = "Dirty";
                        (this.GetInteractable<Bed>() as Bed).Colour = Color.Red;
                        this.StopInteractionWith<Bed>();
                    }
                }
            }

            MacroPathFinder.Update(gameTime);
            MicroPathFinder.Update(gameTime);

            if (MacroPathFinder.IsFollowingPath || MicroPathFinder.IsFollowingPath)
            {
                if (CurrentAnimationName != "Walk")
                {
                    SetAnimation("Walk", 0.0f, 1.2f);
                }
            }
            else
            {
                if (!IsInteractingWith<Chair>() && !IsInteractingWith<Bed>() && !IsInteractingWith<Nurse>())
                {
                    if (CurrentAnimationName != "Idle")
                    {
                        SetAnimation("Idle", 0.0f, 1.4f);
                    }
                }
            }

            // If the macro path has been found then begin to calculate a path 
            // from the last node of the macro path to the exact chair location using the micro path finder
            if (mCurrentState != "Leaving")
            {
                if (MacroPathFinder.HasPathBeenFound)
                {
                    if (!MicroPathFinder.FindingPath && !MicroPathFinder.IsFollowingPath && !MicroPathFinder.HasPathBeenFound)
                    {
                        MicroPathFinder.FindPathFromTo(new Vector3(MacroPathFinder.LastNode.Position.X, 0, MacroPathFinder.LastNode.Position.Y), mChairs[mChair].Position);
                    }
                }
            }

            // Try and sit down in a chair if one has been selected and if it's not interacting with the player
            if (!IsInteractingWith<Nurse>() && !IsInteractingWith<Bed>() && mCurrentState == "Waiting")
            {
                // All of this stuff is relevant only if the patient is not sitting on (interacting with) a chair
                if (!IsInteractingWith<Chair>())
                {
                    if (mChair != -1)
                    {
                        // Force patient to pick a new chair if someone has taken the one they were going for
                        if (mChairs[mChair].IsInteractingWith<Patient>())
                        {
                            MacroPathFinder.CancelCurrentPath();
                            MicroPathFinder.CancelCurrentPath();
                            mChair = -1;
                            PickAChair();
                        }
                        // If no longer following the path and close enough to target chair then interact with
                        else if (BoundingPolygon.Intersects(ref mChairs[mChair].BoundingPolygon) && !MacroPathFinder.IsFollowingPath && !MacroPathFinder.FindingPath)
                        {                           
                            // Interact with chair and continue depending on the sucess of the interaction
                            if (!InteractWith(mChairs[mChair]))
                            {
                                // Interaction failed, chair is taken
                                // Search for another empty chair, if none available just pick a spot nearby to stand
                                MacroPathFinder.CancelCurrentPath();
                                MicroPathFinder.CancelCurrentPath();
                                mChair = -1;
                                PickAChair();
                            }
                            else
                            {
                                // Snap patient to chair
                                Position = mChairs[mChair].Position;

                                Rotation.Y = mChairs[mChair].Rotation.Y;


                                //Model = ContentCache.GetModel("Models\\Animated\\LadySit");
                                SetAnimation("Sit", 0.0f, 1.4f);

                                // DEBUG
                                /*mCurrentState = "Leaving";
                                mSuccessfullyTreated = false;*/

                                MacroPathFinder.CancelCurrentPath();
                                MicroPathFinder.CancelCurrentPath();
                            }

                        }
                        // If no longer following the path and not quite close enough to the chair, switch to the micro path finder to get to the exact position of the chair
                        // YO LOOK AT THIS DAWG IT DONT WORK QUITE RIGHT INNIT
                        else if (!BoundingPolygon.Intersects(ref mChairs[mChair].BoundingPolygon) && !MacroPathFinder.IsFollowingPath && !MacroPathFinder.FindingPath)
                        {
                            MacroPathFinder.CancelCurrentPath();

                            if (MicroPathFinder.HasPathBeenFound)
                            {
                                MicroPathFinder.FollowPath();
                            }
                            else
                            {
                                MicroPathFinder.AutoFollowPath = true;
                                if (!MicroPathFinder.FindingPath && !MicroPathFinder.IsFollowingPath)
                                {
                                    MicroPathFinder.FindPathTo(mChairs[mChair].Position);
                                }
                            }
                        }
                    }
                    else
                    {
                        // Clearly didn't find a chair when spawned so try again
                        PickAChair();
                    }
                }
            }

            Continue(gameTime);
        }

        public void PickAChair()
        {
            float smallestDistance = -1;
            bool chairFound = false;

            for (int i = 0; i < mChairs.Count; i++)
            {
                Chair chair = mChairs[i];

                //if (mChair == -1)
                {
                    if (!chair.IsInteractingWith<Patient>())
                    {
                        float dist = Vector3.DistanceSquared(Position, chair.Position);

                        if (dist < smallestDistance || smallestDistance == -1)
                        {
                            smallestDistance = dist;
                            mGoingBackToSpawnPoint = false;
                            chairFound = true;
                            mChair = i;
                        }
                    }
                }
            }

            if (chairFound)
            {
                MacroPathFinder.CancelCurrentPath();
                MicroPathFinder.CancelCurrentPath();
                MacroPathFinder.FindPathTo(mChairs[mChair].Position);

                // Set urgency timer properly here, not when spawned
                mUrgencyTimer = mRandom.Next(0, (int)mMedUrgencyBound);
            }
            else
            {
                // No chair found so leave
                if (!mGoingBackToSpawnPoint)
                {
                    MacroPathFinder.CancelCurrentPath();
                    mGoingBackToSpawnPoint = true;

                    // GO BACK FROM WHENCE YOU CAME! YOU WERE TOO LATE FOR A CHAIR!
                    MacroPathFinder.FindPathTo(SpawnPos);
                }
            }
        }

        private void SetUrgencyColor()
        {
            if (mUrgencyTimer < mLowUrgencyBound)
            {
                mUrgencyColor = Color.LightGreen;
            }
            else if (mUrgencyTimer > mLowUrgencyBound && mUrgencyTimer < mMedUrgencyBound)
            {
                mUrgencyColor = Color.Orange;
            }
            else if (mUrgencyTimer > mMedUrgencyBound && mUrgencyTimer < mHighUrgencyBound)
            {
                mUrgencyColor = Color.Red;
            }
        }
    }
}
