﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using TotallyRadEngine10;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework;
using PolygonCollisionLibrary;
using System.IO;
using Microsoft.Xna.Framework.Graphics;

namespace ProjectMaternity
{
    class PlayState : State
    {
        public int LevelNumber = 0;
        protected bool mDebug = false;
        protected bool mDebugCam = false;
        protected bool mPlayerInteractionFound = false;
        protected int mNumberOfPatientsSpawned = 1;     // to be used to calculate the mortality rates.       
        protected bool mInteractedWithBed = false;      // stops you from interacting with the cabinets at the same time as the bed. 

        protected int mSpawnRate;
        protected int mSpawnTimer = 100;
        protected TimeSpan mTimer = new TimeSpan(0, 3, 0);

        protected Player mPlayer;
        protected AIManager mAIManager;
        protected List<Patient> mPatients;
        protected List<Chair> mChairs;
        protected List<Bed> mBeds;
        protected List<Cabinet> mCabinets;
        protected List<Renderable> mModels;

        protected List<Polygon> mLevelCollisionPolygons;
        protected List<Polygon> mPathfindingCollision;
        Polygon mPlayerDoorPoly;

        public static bool Playable = true;
        public static int TreatedPatientCount = 430;
        public static int BirthComplications = 320;
        public static int MaternalDeaths = 570;
        public static int DeceasedPatients = 0;
        public static int ObjectiveRate;


        public static int ComplicationChance = 0; // In percents

        //Renderable mCurtainsModel;

        protected bool mPaused = false;

        public override void Load(PersistentData persistent)
        {
            Renderer.LightDir = Vector4.Normalize(new Vector4(-1.0f, -1.5f, 0.5f, 0.0f));

            AudioPlayer.StopAll();

            if (LevelNumber == 0)
            {
                //mTimer = new TimeSpan(0, 10000, 10);
                AudioPlayer.PlaySound("Level1Music");

                AINurse.ResolveComplications = true;
            }
            else if (LevelNumber == 1)
            {
                //AudioPlayer.PlaySound("Level2Music");
                AINurse.ResolveComplications = false;
            }

            Vector2[] colVerts = new Vector2[4] {new Vector2(-10.0f, 34.0f),
                                                             new Vector2(-10.0f, 28.0f),
                                                             new Vector2(10.0f, 28.0f),
                                                             new Vector2(10.0f, 34.0f)};
            mPlayerDoorPoly = new Polygon(colVerts, true);

            // Create the player
            mPlayer = new Player();
            mPlayer.Model = ContentCache.GetModel("Models\\Animated\\NurseIdle1");  // load a model for the player
            mPlayer.Position.Z += 10;
            mPlayer.Scale = new Vector3(0.425f, 0.425f, 0.425f);
            mPlayer.IsCellShaded = true;
            mPlayer.InitialiseAnimation("Content\\Models\\Animated\\NurseAnims.tra");
            mPlayer.SetAnimation("Idle", 0, 1.0f);

            mPatients = new List<Patient>();
            mChairs = new List<Chair>();
            mBeds = new List<Bed>();
            mCabinets = new List<Cabinet>();
            mModels = new List<Renderable>();

            mUI.Add("Bubbles", new BubblesUI());
            mUI.Add("Gameplay", new GameplayUI());
            mUI.Add("PauseMenu", new PauseMenuUI());
            mUI["PauseMenu"].Visible = false;

            // Load entities and level collision data from file
            LoadLevelDataFromFile();

            // Give beds references to their nearest cabinets
            CalculateCabinetsNearestBeds();

            mBackground.IsCellShaded = false;
            mBackground.CastsShadow = false;

            // Provide list of entities for AI Nurses
            mAIManager = new AIManager(mPatients, mBeds, mCabinets);
            //mAIManager.Enabled = false;

            (mUI["Gameplay"] as GameplayUI).InitialMortality = ((float)(MaternalDeaths + DeceasedPatients) /
                (TreatedPatientCount + MaternalDeaths + DeceasedPatients));
            (mUI["Gameplay"] as GameplayUI).ObjectiveRate = ObjectiveRate;
        }

        private void CalculateCabinetsNearestBeds()
        {
            for (int i = 0; i < mBeds.Count; i++)
            {
                Bed bed = mBeds[i];
                float nearestMedicine = float.MaxValue;
                float nearestLaundry = float.MaxValue;
                float newDistance = 0;

                foreach (Cabinet cab in mCabinets)
                {
                    newDistance = Vector3.Distance(bed.Position, cab.Position);

                    if (cab.CabinetType == CabinetType.Medicine)
                    {
                        if (newDistance < nearestMedicine)
                        {
                            bed.NearestCabMed = cab;
                            nearestMedicine = newDistance;
                        }
                    }
                    else if (cab.CabinetType == CabinetType.Laundry)
                    {
                        if (newDistance < nearestLaundry)
                        {
                            bed.NearestCabLaundry = cab;
                            nearestLaundry = newDistance;
                        }
                    }
                }
            }
        }

        private void LoadLevelDataFromFile()
        {
            // Load entities and level collision data from file
            StreamReader reader = new StreamReader("Content\\Levels\\" + LevelNumber.ToString() + ".level");

            string fileLine = reader.ReadLine();

            // Read in initial stats data
            Playable = bool.Parse(fileLine);
            fileLine = reader.ReadLine();
            TreatedPatientCount = int.Parse(fileLine);
            fileLine = reader.ReadLine();
            DeceasedPatients = int.Parse(fileLine);
            fileLine = reader.ReadLine();
            MaternalDeaths = int.Parse(fileLine);
            fileLine = reader.ReadLine();
            BirthComplications = int.Parse(fileLine);

            // Read in level variables
            fileLine = reader.ReadLine();
            ComplicationChance = int.Parse(fileLine);
            fileLine = reader.ReadLine();
            mSpawnRate = int.Parse(fileLine);
            fileLine = reader.ReadLine();
            ObjectiveRate = int.Parse(fileLine);

            reader.ReadLine(); // Ignore model line for now?
            mBackground.Model = ContentCache.GetModel("Models\\Levels\\" + LevelNumber.ToString() + "\\" + LevelNumber.ToString());

            mBackground.Scale = new Vector3(4.00f, 4.00f, 4.00f);

            // Read in entities
            fileLine = reader.ReadLine();
            while (fileLine != "CollisionData")
            {
                switch (fileLine)
                {
                    case "PlayerSpawnEnt":
                        {
                            float playerXPos = float.Parse(reader.ReadLine());
                            float playerYPos = float.Parse(reader.ReadLine());
                            float playerZPos = float.Parse(reader.ReadLine());
                            float playerRot = float.Parse(reader.ReadLine());
                            float playerScale = float.Parse(reader.ReadLine());

                            mPlayer.Position.X = playerXPos;
                            mPlayer.Position.Y = playerYPos;
                            mPlayer.Position.Z = playerZPos;
                            mPlayer.Rotation.Y = playerRot;
                            /* mPlayer.Scale.X += playerScale;
                             mPlayer.Scale.Y += playerScale;
                             mPlayer.Scale.Z += playerScale;*/
                            mPlayer.IsCellShaded = true;

                            break;
                        }
                    case "BedEnt":
                        {
                            float entXPos = float.Parse(reader.ReadLine());
                            float entYPos = float.Parse(reader.ReadLine());
                            float entZPos = float.Parse(reader.ReadLine());
                            float entRot = float.Parse(reader.ReadLine());
                            float entScale = float.Parse(reader.ReadLine());

                            Bed bed = new Bed();

                            bed.Position.X = entXPos;
                            //bed.Position.X = 10000;
                            bed.Position.Y = entYPos;
                            bed.Position.Z = entZPos;
                            bed.Rotation.Y = entRot;
                            bed.Scale.X += entScale;
                            bed.Scale.Y += entScale;
                            bed.Scale.Z += entScale;
                            bed.IsCellShaded = true;

                            mBeds.Add(bed);
                            mBeds[mBeds.Count - 1].Model = ContentCache.GetModel("Models\\Static\\Bed");

                            break;
                        }
                    case "ChairEnt":
                        {
                            float entXPos = float.Parse(reader.ReadLine());
                            float entYPos = float.Parse(reader.ReadLine());
                            float entZPos = float.Parse(reader.ReadLine());
                            float entRot = float.Parse(reader.ReadLine());
                            float entScale = float.Parse(reader.ReadLine());

                            Chair chair = new Chair();

                            chair.Model = ContentCache.GetModel("Models\\Static\\Chair");

                            chair.Position.X = entXPos;
                            chair.Position.Y = entYPos;
                            chair.Position.Z = entZPos;
                            chair.Rotation.Y = entRot;
                            chair.Scale.X += entScale;
                            chair.Scale.Y += entScale;
                            chair.Scale.Z += entScale;
                            chair.IsCellShaded = true;

                            mChairs.Add(chair);

                            break;
                        }
                    case "MedCabEnt":
                        {
                            float entXPos = float.Parse(reader.ReadLine());
                            float entYPos = float.Parse(reader.ReadLine());
                            float entZPos = float.Parse(reader.ReadLine());
                            float entRot = float.Parse(reader.ReadLine());
                            float entScale = float.Parse(reader.ReadLine());

                            Cabinet cab = new Cabinet(CabinetType.Medicine);

                            cab.Model = ContentCache.GetModel("Models\\Static\\OpMetalTable");

                            cab.Position.X = entXPos;
                            cab.Position.Y = entYPos;
                            cab.Position.Z = entZPos;
                            cab.Rotation.Y = entRot;
                            cab.Scale.X += entScale;
                            cab.Scale.Y += entScale;
                            cab.Scale.Z += entScale;
                            cab.IsCellShaded = true;

                            mCabinets.Add(cab);
                            break;
                        }
                    case "SheetsCabEnt":
                        {
                            float entXPos = float.Parse(reader.ReadLine());
                            float entYPos = float.Parse(reader.ReadLine());
                            float entZPos = float.Parse(reader.ReadLine());
                            float entRot = float.Parse(reader.ReadLine());
                            float entScale = float.Parse(reader.ReadLine());

                            Cabinet cab = new Cabinet(CabinetType.Laundry);

                            cab.Model = ContentCache.GetModel("Models\\Static\\SheetsCupboard");

                            cab.Position.X = entXPos;
                            cab.Position.Y = entYPos;
                            cab.Position.Z = entZPos;
                            cab.Rotation.Y = entRot;
                            cab.Scale.X += entScale;
                            cab.Scale.Y += entScale;
                            cab.Scale.Z += entScale;
                            cab.IsCellShaded = true;

                            mCabinets.Add(cab);
                            break;
                        }
                    default:
                        {
                            if (fileLine.StartsWith("Model: "))
                            {
                                string modelName = fileLine.Substring(7);
                                float entXPos = float.Parse(reader.ReadLine());
                                float entYPos = float.Parse(reader.ReadLine());
                                float entZPos = float.Parse(reader.ReadLine());
                                float entRot = float.Parse(reader.ReadLine());
                                float entScale = float.Parse(reader.ReadLine());

                                Renderable model = new Renderable();

                                model.Model = ContentCache.GetModel("Models\\Static\\" + modelName.Split('.')[0]);

                                model.Position.X = entXPos;
                                model.Position.Y = entYPos;
                                model.Position.Z = entZPos;
                                model.Rotation.Y = entRot;
                                model.Scale.X += entScale;
                                model.Scale.Y += entScale;
                                model.Scale.Z += entScale;

                                mModels.Add(model);
                            }

                            break;
                        }
                }

                fileLine = reader.ReadLine();
            }

            // Read in collision data
            mLevelCollisionPolygons = new List<Polygon>();

            bool readingPoly = false;
            List<Vector2> currentPoly = new List<Vector2>();

            do
            {
                fileLine = reader.ReadLine();

                if (readingPoly == false && fileLine == "CollisionPoly")
                {
                    readingPoly = true;
                }
                else if (readingPoly == true && fileLine == "EndCollisionPoly")
                {
                    mLevelCollisionPolygons.Add(new Polygon(currentPoly.ToArray(), true));
                    currentPoly.Clear();
                    readingPoly = false;
                }
                else if (readingPoly == true && fileLine != "EndCollisionPoly")
                {
                    float polyX = float.Parse(fileLine);
                    float polyY = float.Parse(reader.ReadLine());

                    currentPoly.Add(new Vector2(polyX, polyY));
                }
            } while (fileLine != "End");

            // End of file
            reader.Close();

            // Pathfinding collision on the macro level
            mPathfindingCollision = new List<Polygon>();
            mPathfindingCollision.AddRange(mLevelCollisionPolygons);

            // This might cause problems
            foreach (Bed bed in mBeds)
            {
                bed.Update(new GameTime());
                mPathfindingCollision.Add(bed.BoundingPolygon);
            }

            /*foreach (Cabinet cabinet in mCabinets)
            {
                if (cabinet.CabinetType == CabinetType.Laundry)
                {
                    cabinet.Update(new GameTime());
                    mPathfindingCollision.Add(cabinet.BoundingPolygon);
                }
            }   

            foreach (Chair chair in mChairs)
            {
                chair.Update(new GameTime());
                mPathfindingCollision.Add(chair.BoundingPolygon);
            }*/

            MacroPathFinder.CreateEnvironmentMap(mPathfindingCollision);

            // Pathfinding collision on the micro level
            mPathfindingCollision = new List<Polygon>();

            mPathfindingCollision.AddRange(mLevelCollisionPolygons);

            // This might cause problems
            foreach (Bed bed in mBeds)
            {
                bed.Update(new GameTime());
                mPathfindingCollision.Add(bed.BoundingPolygon);
            }

            /*foreach (Cabinet cabinet in mCabinets)
            {
                if (cabinet.CabinetType == CabinetType.Laundry)
                {
                    cabinet.Update(new GameTime());
                    mPathfindingCollision.Add(cabinet.BoundingPolygon);
                }
            }  */

            mPathfindingCollision.AddRange(mLevelCollisionPolygons);


            MicroPathFinder.CreateEnvironmentMap(mPathfindingCollision);

            mSceneCamera.Reset(mPlayer.Position);
            mSceneCamera.RotateRelX(50);
            mSceneCamera.RotateY(95);
            mSceneCamera.MoveForward(-50);
        }

        void HighlightInteraction()
        {
            for (int p = 0; p < mPatients.Count; p++)
            {
                Patient patient = mPatients[p];

                // Highlight a patient that is in interaction range 
                if (!patient.IsInteractingWith<Bed>() && !patient.IsInteractingWith<Player>() && patient.CurrentState != "Leaving")
                {
                    patient.Colour = Color.White;

                    if (!mPlayerInteractionFound)
                    {
                        if (mPlayer.InteractionPoly.Intersects(ref patient.BoundingPolygon))
                        {
                            patient.Colour = Color.Blue;
                            mPlayerInteractionFound = true;
                        }
                    }
                }
            }

            // Highlight a bed that is in interaction range 
            foreach (Bed bed in mBeds)
            {
                bed.Colour = Color.White; // Reset bed color

                if (bed.CurrentState == "Dirty")
                {
                    bed.Colour = Color.Red;
                }

                if (!mPlayerInteractionFound)
                {
                    if (mPlayer.InteractionPoly.Intersects(ref bed.BoundingPolygon))
                    {
                        bed.Colour = Color.Blue;
                        mPlayerInteractionFound = true;
                    }
                }
            }

            // Highlight a cabinet that is in interaction range
            foreach (Cabinet cab in mCabinets)
            {
                cab.Colour = Color.White;

                if (!mPlayerInteractionFound)
                {
                    if (mPlayer.InteractionPoly.Intersects(ref cab.BoundingPolygon))
                    {
                        cab.Colour = Color.Blue;
                        mPlayerInteractionFound = true;
                    }
                }
            }
        }

        void CheckForPlayerInteraction()
        {
            if (!mPlayer.IsInteractingWith<Bed>())
            {
                // Check for interaction between player and patients
                foreach (Patient patient in mPatients)
                {
                    if (mPlayer.InteractionPoly.Intersects(ref patient.BoundingPolygon))
                    {
                        //patient.Colour = Color.White;
                        mPlayer.InteractWith(patient);
                        break;
                    }
                }

                // Check for interaction between player and beds
                foreach (Bed bed in mBeds)
                {
                    mInteractedWithBed = false;

                    if (mPlayer.InteractionPoly.Intersects(ref bed.BoundingPolygon))
                    {
                        mPlayer.InteractWith(bed);  // Only interact with one bed at a time
                        mInteractedWithBed = true;
                        break;
                    }
                }

                // Check for interaction between player and medicine cabinet
                foreach (Cabinet cabinet in mCabinets)
                {
                    if (!mInteractedWithBed && mPlayer.InteractionPoly.Intersects(ref cabinet.BoundingPolygon))
                    {
                        // Pick up new item
                        if (mPlayer.CurrentItem == Item.None)
                        {
                            mPlayer.InteractWith(cabinet);
                            break;
                        }
                        // Put back dirty sheets
                        else if (mPlayer.CurrentItem == Item.DirtySheets && cabinet.CabinetType == CabinetType.Laundry)
                        {
                            mPlayer.InteractWith(cabinet);
                            break;
                        }
                        // Put back extra medical equipment
                        else if (mPlayer.CurrentItem == Item.Medical && cabinet.CabinetType == CabinetType.Medicine)
                        {
                            mPlayer.CurrentItem = Item.None;
                            break;
                        }
                        // Put back extra clean sheets
                        else if (mPlayer.CurrentItem == Item.CleanSheets && cabinet.CabinetType == CabinetType.Laundry)
                        {
                            mPlayer.CurrentItem = Item.None;
                            break;
                        }
                    }
                }

                // Put extra items back in cabinets
                foreach (Cabinet cabinet in mCabinets)
                {
                    if (mPlayer.InteractionPoly.Intersects(ref cabinet.BoundingPolygon))
                    {

                    }
                }
            }
        }

        void CheckForPlayerSecondaryInteraction()
        {
            // Release patient
            if (mPlayer.IsInteractingWith<Patient>())
            {
                (mPlayer.GetInteractable<Patient>() as Patient).PickAChair();
                mPlayer.StopInteractionWith<Patient>();
            }
        }

        public override void Update(Microsoft.Xna.Framework.GameTime gameTime)
        {
            if ((InputManager.HasButtonBeenPressed(Keys.Escape) ||
                InputManager.HasButtonBeenPressed(Buttons.Start)) &&
                !(this is TutorialState))
            {
                mPaused = !mPaused;
                mUI["PauseMenu"].Visible = !mUI["PauseMenu"].Visible;
            }

            if (!mPaused)
            {
                mPlayer.ResumeAnimation();
                foreach (AINurse nurse in mAIManager.mAINurses)
                {
                    nurse.ResumeAnimation();
                }
                foreach (Patient patient in mPatients)
                {
                    patient.ResumeAnimation();
                }

                // Update the game timer
                mTimer -= gameTime.ElapsedGameTime;

                if (mTimer <= TimeSpan.Zero)
                {
                    // Game over!
                    StateManager.SetState(new EndLevelState());
                }

                // Update the player
                mPlayer.Update(gameTime);
                mPlayerInteractionFound = false;

                // Update the UI data
                (mUI["Bubbles"] as BubblesUI).Patients = mPatients;
                (mUI["Gameplay"] as GameplayUI).Timer = mTimer;
                (mUI["Gameplay"] as GameplayUI).PatientsTreated = TreatedPatientCount;
                (mUI["Gameplay"] as GameplayUI).ComplicationCount = BirthComplications;
                (mUI["Gameplay"] as GameplayUI).MaternalMortality = MaternalDeaths;
                (mUI["Gameplay"] as GameplayUI).OnCallShiftActive = mAIManager.ShiftActive;
                (mUI["Gameplay"] as GameplayUI).ShiftTimer = mAIManager.ShiftTimer;
                (mUI["Gameplay"] as GameplayUI).MaxShiftTimer = mAIManager.MaxShiftTimer;
                (mUI["Gameplay"] as GameplayUI).OnCallAvailable = mAIManager.OnCallAvailable;
                (mUI["Gameplay"] as GameplayUI).OnCallTimer = mAIManager.OnCallTimer;
                (mUI["Gameplay"] as GameplayUI).MaxOnCallTimer = mAIManager.MaxOnCallTimer;
                (mUI["Gameplay"] as GameplayUI).OnCallLevel = mAIManager.OnCallLevel;
                (mUI["Gameplay"] as GameplayUI).PatientsDeceased = DeceasedPatients;


                bool collidingWithLevel = false;

                // Check for collision against the level geometry
                for (int i = 0; i < mLevelCollisionPolygons.Count; i++)
                {
                    Polygon colPoly = mLevelCollisionPolygons[i];
                    if (mPlayer.BoundingPolygon.Intersects(ref colPoly))
                    {
                        mPlayer.Position += new Vector3(mPlayer.BoundingPolygon.collisionData.MTD.X, 0,
                                                        mPlayer.BoundingPolygon.collisionData.MTD.Y);

                        collidingWithLevel = true;
                    }
                }

                // Can't go outside
                if (mPlayer.BoundingPolygon.Intersects(ref mPlayerDoorPoly))
                {
                    mPlayer.Position += new Vector3(mPlayer.BoundingPolygon.collisionData.MTD.X, 0,
                                                       mPlayer.BoundingPolygon.collisionData.MTD.Y);

                    collidingWithLevel = true;
                }


                // Highlight any potential interactions betwene the player and level entities
                if (!collidingWithLevel)
                {
                    HighlightInteraction();
                }

                // Update the AI Nurses
                mAIManager.Update(gameTime, mPlayer);

                // Update all patients
                // Highlight entities that the AI is targetting
                foreach (Patient patient in mPatients)
                {
                    patient.IsCellShaded = true;
                    patient.Update(gameTime);
                    CollisionManager.Resolve(patient, mPlayer);
                    //CollisionManager.ResolveList(patient, mAIManager.mAINurses);

                    if (patient.TargetedBy != null)
                    {
                        //patient.Colour = Color.Green;
                    }
                }

                // Update all chairs
                foreach (Chair chair in mChairs)
                {
                    chair.Update(gameTime);
                    CollisionManager.Resolve(chair, mPlayer);
                    //CollisionManager.ResolveList(chair, mAIManager.mAINurses);
                    //CollisionManager.ResolveList(chair, mPatients);

                    if (chair.TargetedBy != null)
                    {
                        //chair.Colour = Color.Green;
                    }
                }

                // Update all beds
                foreach (Bed bed in mBeds)
                {
                    bed.Update(gameTime);
                    CollisionManager.Resolve(bed, mPlayer);
                    //CollisionManager.ResolveList(bed, mAIManager.mAINurses);
                    //CollisionManager.ResolveList(bed, mPatients);

                    if (bed.TargetedBy != null)
                    {
                        //bed.Colour = Color.Green;
                    }
                }

                // Update all cabinets
                foreach (Cabinet cabinet in mCabinets)
                {
                    cabinet.Update(gameTime);
                    CollisionManager.Resolve(cabinet, mPlayer);
                    //CollisionManager.ResolveList(cabinet, mAIManager.mAINurses);
                    //CollisionManager.ResolveList(cabinet, mPatients);

                    if (cabinet.TargetedBy != null)
                    {
                        //cabinet.Colour = Color.Green;
                    }
                }

                // Patient spawning
                if (mSpawnTimer > 0)
                {
                    mSpawnTimer--;
                }
                else if (mSpawnTimer <= 0)
                {
                    Random random = new Random();

                    mNumberOfPatientsSpawned++;
                    mPatients.Add(new Patient());
                    mPatients[mPatients.Count - 1].Position.X = -1.0f;
                    mPatients[mPatients.Count - 1].Position.Z = 32.0f;
                    mPatients[mPatients.Count - 1].SpawnPos = mPatients[mPatients.Count - 1].Position;
                    mPatients[mPatients.Count - 1].MacroPathFinder.AutoFollowPath = true;
                    mPatients[mPatients.Count - 1].SetChairs(ref mChairs);
                    mPatients[mPatients.Count - 1].PickAChair();
                    mPatients[mPatients.Count - 1].UniqueID = mNumberOfPatientsSpawned;

                    mSpawnTimer = mSpawnRate;
                }


                // Toggle debug mode
                if (InputManager.HasButtonBeenPressed(Keys.F1))
                {
                    mDebug = !mDebug;
                }

                if (InputManager.HasButtonBeenPressed(Keys.F2))
                {
                    mDebugCam = !mDebugCam;
                }

                for (int p = 0; p < mPatients.Count; p++)
                {
                    Patient patient = mPatients[p];

                    // Update game stats
                    if (!patient.IsAlive)
                    {
                        DeceasedPatients++;
                        mPatients.Remove(patient);
                        mUI["Bubbles"].RemoveImage("Bubble" + patient.UniqueID.ToString());
                    }

                    if (patient.HasLeft && !patient.SuccesfullyTreated)
                    {
                        mPatients.Remove(patient);
                        mUI["Bubbles"].RemoveImage("Bubble" + patient.UniqueID.ToString());
                    }
                    else if (patient.HasLeft)
                    {
                        mPatients.Remove(patient);
                        mUI["Bubbles"].RemoveImage("Bubble" + patient.UniqueID.ToString());
                    }
                }

                // Initialise an interaction when the action buton is pressed
                if ((InputManager.HasButtonBeenPressed(Keys.Space) || InputManager.HasButtonBeenPressed(Buttons.A))
                    && mPlayer.CanMove)
                {
                    if (!collidingWithLevel)
                    {
                        CheckForPlayerInteraction();
                    }
                }

                // Cancel interaction/perform secondary interaction when the secondary button is pressed
                if (InputManager.HasButtonBeenPressed(Keys.LeftAlt) || InputManager.HasButtonBeenPressed(Keys.LeftControl) ||
                    InputManager.HasButtonBeenPressed(Keys.LeftShift) || InputManager.HasButtonBeenPressed(Buttons.B))
                {
                    CheckForPlayerSecondaryInteraction();
                }

                // Update the camera to follow the player
                if (!mDebugCam)
                {
                    mSceneCamera.Reset(mPlayer.Position);
                    mSceneCamera.RotateRelX(50);
                    mSceneCamera.RotateY(95);
                    mSceneCamera.MoveForward(-50);
                    //mSceneCamera.SetLookTarget(mPlayer.Position);
                }
                else
                {
                    #region Move Debug Camera

                    if (InputManager.IsButtonDown(Keys.W))
                    {
                        mSceneCamera.MoveForward(0.5f);
                    }

                    if (InputManager.IsButtonDown(Keys.S))
                    {
                        mSceneCamera.MoveForward(-0.5f);
                    }

                    if (InputManager.IsButtonDown(Keys.A))
                    {
                        mSceneCamera.MoveLeft(0.5f);
                    }

                    if (InputManager.IsButtonDown(Keys.D))
                    {
                        mSceneCamera.MoveLeft(-0.5f);
                    }

                    if (InputManager.IsButtonDown(Keys.Q))
                    {
                        mSceneCamera.RotateY(0.5f);
                    }

                    if (InputManager.IsButtonDown(Keys.E))
                    {
                        mSceneCamera.RotateY(-0.5f);
                    }

                    if (InputManager.IsButtonDown(Keys.R))
                    {
                        mSceneCamera.RotateRelX(-0.5f);
                    }

                    if (InputManager.IsButtonDown(Keys.F))
                    {
                        mSceneCamera.RotateRelX(0.5f);
                    }

                    #endregion
                }



            }
            else
            {
                mPlayer.PauseAnimation();
                foreach (AINurse nurse in mAIManager.mAINurses)
                {
                    nurse.PauseAnimation();
                }
                foreach (Patient patient in mPatients)
                {
                    patient.PauseAnimation();
                }
            }

            // Update the UI
            foreach (KeyValuePair<string, UI> ui in mUI)
            {
                ui.Value.Update();
            }
        }

        public override void Draw(Renderer renderer, GameTime gameTime)
        {
            renderer.Set3DStates();
            renderer.SetCamera(mSceneCamera);

            // Draw the level geometry
            renderer.DrawRenderable(mBackground, gameTime);

            if (mDebug)
            {
                MacroPathFinder.DrawEnvironmentMap(renderer);
                //MicroPathFinder.DrawEnvironmentMap(renderer);

                foreach (Polygon poly in mLevelCollisionPolygons)
                {
                    poly.RenderDataColor = Color.White;
                    renderer.AddPolygonToDraw(poly);
                }

                foreach (Patient patient in mPatients)
                {
                    //patient.MacroPathFinder.DrawPath(renderer);
                    //patient.MicroPathFinder.DrawPath(renderer);
                }

                foreach (AINurse aiNurse in mAIManager.mAINurses)
                {
                    aiNurse.MacroPathFinder.DrawPath(renderer);
                    aiNurse.MicroPathFinder.DrawPath(renderer);
                }
            }

            // Update nurse models based on what they're doing
            mAIManager.Draw(gameTime);

            renderer.DrawRenderable(mPlayer, gameTime);
            if (mPlayer.CurrentItem != Item.None && !mPlayer.IsInteractingWith<Bed>())
            {
                renderer.DrawRenderable(mPlayer.ItemRenderable, gameTime);
            }

            // Draw the level entities
            foreach (Chair chair in mChairs)
            {
                renderer.DrawRenderable(chair, gameTime);
            }

            foreach (Bed bed in mBeds)
            {
                renderer.DrawRenderable(bed, gameTime);
            }

            foreach (Cabinet cabinet in mCabinets)
            {
                renderer.DrawRenderable(cabinet, gameTime);
            }

            foreach (Renderable model in mModels)
            {
                renderer.DrawRenderable(model, gameTime);
            }

            for (int i = 0; i < mPatients.Count; i++)
            {
                Patient patient = mPatients[i];
                renderer.DrawRenderable(patient, gameTime);
            }

            foreach (AINurse aiNurse in mAIManager.mAINurses)
            {
                renderer.DrawRenderable(aiNurse, gameTime);
                if (aiNurse.CurrentItem != Item.None && !aiNurse.IsInteractingWith<Bed>())
                {
                    renderer.DrawRenderable(aiNurse.ItemRenderable, gameTime);
                }
            }

            // Draw all polygons
            renderer.DrawPolygonBatch();

            // Apply post-processing to the scene and present it to the main display
            renderer.PresentScene();

            // Update UI data
            (mUI["Bubbles"] as BubblesUI).ViewMatrix = renderer.ViewMatrix;
            (mUI["Bubbles"] as BubblesUI).ProjectionMatrix = renderer.ProjectionMatrix;

            // Draw the UI
            if (!mDebugCam)
            {
                foreach (KeyValuePair<string, UI> ui in mUI)
                {
                    ui.Value.Draw(renderer.SpriteBatch);
                }
            }
        }

        public override void Unload(PersistentData persistent)
        {
            // Level over
            persistent.Integers["LevelNo"] = LevelNumber;
            // Store stats in persistent data so player can see them in the End Level state
            persistent.Integers["PatientsTreated"] = TreatedPatientCount;
            persistent.Integers["ComplicationCount"] = BirthComplications;
            persistent.Integers["MaternalDeaths"] = MaternalDeaths;
            persistent.Integers["Improved"] = (int)((float)(((float)TreatedPatientCount - (float)DeceasedPatients) / (float)ObjectiveRate) * 100.0f);//(int)(((MaternalDeaths - (DeceasedPatients) / MaternalDeaths) * 100.0f);
            persistent.Booleans["Failure"] = false;// (TreatedPatientCount < ObjectiveRate);
            persistent.Integers["Objective"] = ObjectiveRate;
        }
    }
}