﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using TotallyRadEngine10;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;

namespace ProjectMaternity
{
    class TutorialState : PlayState
    {
        private Dictionary<string, bool> mStages;   // Made it a dictionary to make things a little easier to follow  
        private string mStage = "Welcome";
        private int mTutTimer = 0;

        public override void Load(PersistentData persistent)
        {
            base.Load(persistent);

            mUI.Add("TutorialDialog", new TutorialDialogUI());
            mUI.Add("Arrow", new TutArrowUI());

            // Show the welcome dialog to begin with
            mPaused = true;
            mUI["TutorialDialog"].Visible = true;
            (mUI["TutorialDialog"] as TutorialDialogUI).Message = "         Welcome to Project Maternity!          \n" +
                                                                  "  Prompts like these will teach you the basics. \n" +
                                                                  "   Your first patient will be arriving shortly. \n";
            mUI["Arrow"].Visible = false;
            mUI["Arrow"].Visible = false;

            // Initialise the tutorial stages
            mStages = new Dictionary<string, bool>();
            mStages.Add("Welcome", false);
            mStages.Add("PatientEntry", false);
            mStages.Add("PatientFollow", false);
            mStages.Add("PatientInBed", false);
            mStages.Add("WaitingForDelivery", false);
            mStages.Add("PatientAssistance", false);
            mStages.Add("PatientComplication", false);
            mStages.Add("ItemPickup", false);
            mStages.Add("PatientResting", false);
            mStages.Add("PatientLeaving", false);
            mStages.Add("CleanBed", false);
            mStages.Add("DirtySheets", false);
            mStages.Add("AINurseIntro", false);
            mStages.Add("NurseShiftWaiting", false);
            mStages.Add("NurseShiftBegin", false);
            mStages.Add("NurseExplanation", false);
            mStages.Add("NurseShiftEnd", false);
            mStages.Add("End", false);

            // Force complication
            ComplicationChance = 100;
        }

        public override void Update(GameTime gameTime)
        {
            base.Update(gameTime);

            // Disallow normal patient spawning by never letting the timer decrease
            mSpawnTimer++;

            // Disable the timer
            mUI["Gameplay"].GetText("TimerClock").String = "----";
            mTimer.Add(new TimeSpan(0, 0, 10));

            // Disable AI shifts until needed
            if (mStages["AINurseIntro"] == false)
            {
                mAIManager.Enabled = false;
            }

            // Allow the tutorial dialogs to be closed
            if (mUI["TutorialDialog"].Visible)
            {
                if (InputManager.HasButtonBeenPressed(Keys.Space) || InputManager.HasButtonBeenPressed(Buttons.A))
                {
                    mUI["TutorialDialog"].Visible = false;
                    mPaused = false;
                }
            }

            if (mStage == "Welcome")
            {
                if (!mUI["TutorialDialog"].Visible)
                {
                    mPaused = false;
                    mStages[mStage] = true;
                    mStage = "PatientEntry";
                }
            }

            if (mStage == "PatientEntry")
            {
                // Spawn a single patient
                if (mPatients.Count == 0)
                {
                    mNumberOfPatientsSpawned++;
                    mPatients.Add(new Patient());
                    mPatients[mPatients.Count - 1].Position.X = -1.0f;
                    mPatients[mPatients.Count - 1].Position.Z = 32.0f;
                    mPatients[mPatients.Count - 1].SpawnPos = mPatients[mPatients.Count - 1].Position;
                    mPatients[mPatients.Count - 1].MacroPathFinder.AutoFollowPath = true;
                    mPatients[mPatients.Count - 1].SetChairs(ref mChairs);
                    mPatients[mPatients.Count - 1].PickAChair();
                    mPatients[mPatients.Count - 1].UniqueID = mNumberOfPatientsSpawned;
                }
                // Wait for the patient to sit down
                else if (mPatients[0].IsInteractingWith<Chair>() && !mStages[mStage])
                {
                    // Show the dialog
                    mPaused = true;
                    mUI["Arrow"].Visible = true;
                    mUI["TutorialDialog"].Visible = true;
                    (mUI["Arrow"] as TutArrowUI).Target = mPatients[0].Position;
                    (mUI["TutorialDialog"] as TutorialDialogUI).Message = "         This patient needs your help!          \n" +
                                                                          "  Move over to her and press (A) to interact.   \n";
                    mStages[mStage] = true;
                }

                if (mStages[mStage] && !mUI["TutorialDialog"].Visible)
                {
                    mPaused = false;
                    mStage = "PatientFollow";
                }
            }

            if (mStage == "PatientFollow" || mStage == "PatientEntry")
            {
                // She aint gonna die whilst the tutorial is running
                mPatients[0].UrgencyTimer = 0;

                if (mPatients[0].IsInteractingWith<Nurse>() && !mStages[mStage])
                {
                    // Show the dialog
                    mPaused = true;
                    mUI["Arrow"].Visible = true;
                    mUI["TutorialDialog"].Visible = true;
                    (mUI["Arrow"] as TutArrowUI).Target = mBeds[2].Position;
                    (mUI["TutorialDialog"] as TutorialDialogUI).Message = "       This patient is now following you.       \n" +
                                                                          "        Take her to an empty bed to rest.       \n" +
                                                                          "       Press (A) to inteact with the bed.       \n" +
                                                                          "         Press (B) to release a patient.        \n";
                    mStages[mStage] = true;
                }

                if (mStages[mStage] && !mUI["TutorialDialog"].Visible)
                {
                    mPaused = false;
                    mStage = "PatientInBed";
                }
            }

            if (mStage == "PatientInBed")
            {
                if (mPatients[0].IsInteractingWith<Bed>() && !mStages[mStage])
                {
                    // Show the dialog
                    mPaused = true;
                    mUI["Arrow"].Visible = false;
                    mUI["TutorialDialog"].Visible = true;
                    (mUI["TutorialDialog"] as TutorialDialogUI).Message = "          This patient is now in bed.           \n" +
                                                                          "  She will remain here until she gives birth.   \n";

                    mStages[mStage] = true;
                }

                if (mStages[mStage] && !mUI["TutorialDialog"].Visible)
                {
                    mPaused = false;
                    mStage = "WaitingForDelivery";
                    mTutTimer = 0;
                }
            }

            if (mStage == "WaitingForDelivery")
            {
                // Wait a bit before informing the player of what they could be doing
                if (!mStages[mStage])
                {
                    mTutTimer++;
                }

                if (mTutTimer == 50)
                {
                    if (!mStages[mStage])
                    {
                        // Show the dialog
                        mPaused = true;
                        mUI["TutorialDialog"].Visible = true;
                        (mUI["TutorialDialog"] as TutorialDialogUI).Message = "           You should never be idle!            \n" +
                                                                              " Use this time in later levels to help others.  \n";

                        mStages[mStage] = true;
                    }

                    if (mStages[mStage] && !mUI["TutorialDialog"].Visible)
                    {
                        mPaused = false;
                        mStage = "PatientAssistance";
                        mTutTimer = 0;
                    }
                }
            }

            if (mStage == "PatientAssistance")
            {
                if (mPatients[0].UrgencyTimer != -1 && !mStages[mStage])
                {
                    // Show the dialog
                    mPaused = true;
                    mUI["Arrow"].Visible = true;
                    mUI["TutorialDialog"].Visible = true;
                    (mUI["Arrow"] as TutArrowUI).Target = mPatients[0].GetInteractable<Bed>().Position;
                    (mUI["TutorialDialog"] as TutorialDialogUI).Message = "     This patient requires your assistance.     \n" +
                                                                          "   Move to her bed and press (A) to interact.   \n";

                    mStages[mStage] = true;
                }

                if (mStages[mStage] && !mUI["TutorialDialog"].Visible)
                {
                    mPaused = false;
                    mStage = "PatientComplication";
                }
            }

            if (mStage == "PatientComplication")
            {
                // She aint gonna die whilst the tutorial is running
                mPatients[0].UrgencyTimer = 0;

                if (mPatients[0].UrgencyTimer != -1 && mPatients[0].CurrentState == "Complicated" && !mStages[mStage])
                {
                    // Show the dialog
                    mPaused = true;
                    mUI["TutorialDialog"].Visible = true;
                    (mUI["Arrow"] as TutArrowUI).Target = mCabinets[0].Position;
                    (mUI["TutorialDialog"] as TutorialDialogUI).Message = "   This patient has developed a complication.   \n" +
                                                                          "     Pick up surgical tools and assist her!     \n" +
                                                                          "                    Quickly!                    \n";

                    mStages[mStage] = true;
                }

                if (mStages[mStage] && !mUI["TutorialDialog"].Visible)
                {
                    mPaused = false;
                    mStage = "ItemPickup";
                }
            }

            if (mStage == "ItemPickup")
            {
                // She aint gonna die whilst the tutorial is running
                mPatients[0].UrgencyTimer = 0;

                if (mPlayer.CurrentItem != Item.None && !mStages[mStage])
                {
                    // Show the arrow to the bed
                    mPaused = true;
                    mUI["TutorialDialog"].Visible = true;
                    (mUI["Arrow"] as TutArrowUI).Target = mPatients[0].GetInteractable<Bed>().Position;
                    (mUI["TutorialDialog"] as TutorialDialogUI).Message = "            You have picked up an item.           \n" +
                                                                          "     Return an unwanted item by pressing (A)      \n" +
                                                                          "        next to the object you got it from        \n";

                    mStages[mStage] = true;
                }

                if (mStages[mStage] && !mUI["TutorialDialog"].Visible)
                {
                    mPaused = false;
                    mStage = "PatientResting";
                }
            }

            if (mStage == "PatientResting")
            {
                if (mPatients[0].UrgencyTimer == -1 && mPatients[0].CurrentState == "Resting" && !mStages[mStage])
                {
                    // Show the dialog
                    mPaused = true;
                    mUI["Arrow"].Visible = false;
                    mUI["TutorialDialog"].Visible = true;
                    (mUI["TutorialDialog"] as TutorialDialogUI).Message = "  This patient has given birth and is resting.  \n" +
                                                                          "             She will leave shortly.            \n";

                    mStages[mStage] = true;
                }

                if (mStages[mStage] && !mUI["TutorialDialog"].Visible)
                {
                    mPaused = false;
                    mStage = "PatientLeaving";
                }
            }

            if (mStage == "PatientLeaving")
            {
                if (mPatients.Count == 0 && !mStages[mStage])
                {
                    // Show the dialog
                    mPaused = true;
                    mUI["Arrow"].Visible = true;
                    (mUI["Arrow"] as TutArrowUI).Target = mCabinets[1].Position;
                    mUI["TutorialDialog"].Visible = true;
                    (mUI["TutorialDialog"] as TutorialDialogUI).Message = "        This bed now needs to be cleaned.       \n" +
                                                                          "  Interact with the cupboard for clean sheets.  \n" +
                                                                          "  Then move to the bed and press (A) to clean.  \n";

                    mStages[mStage] = true;
                }

                if (mStages[mStage] && !mUI["TutorialDialog"].Visible)
                {
                    mPaused = false;
                    mStage = "CleanBed";

                    mTutTimer = 0;
                }
            }

            if (mStage == "CleanBed" || (mPlayer.CurrentItem == Item.DirtySheets && mStage != "DirtySheets" && mNumberOfPatientsSpawned <= 2))
            {
                mStage = "CleanBed";

                if (mPlayer.IsInteractingWith<Bed>() && !mStages[mStage])
                {
                    // Show the dialog
                    mPaused = true;
                    mUI["Arrow"].Visible = true;
                    (mUI["Arrow"] as TutArrowUI).Target = mCabinets[1].Position;
                    mUI["TutorialDialog"].Visible = true;
                    (mUI["TutorialDialog"] as TutorialDialogUI).Message = " You must now dispose of the old sheets. \n" +
                                                                          "   Interact with the cupboard to do so.  \n";

                    mStages[mStage] = true;
                }

                if (mStages[mStage] && !mUI["TutorialDialog"].Visible)
                {
                    mPaused = false;
                    mStage = "DirtySheets";

                    mTutTimer = 0;
                }
            }

            if (mStage == "DirtySheets")
            {
                if (mPlayer.CurrentItem == Item.None && !mStages[mStage])
                {
                    // Show the dialog
                    mPaused = true;
                    mUI["Arrow"].Visible = false;
                    mUI["TutorialDialog"].Visible = true;
                    (mUI["TutorialDialog"] as TutorialDialogUI).Message = "             Congratulations!            \n" +
                                                                          "   You have treated your first patient.  \n" +
                                                                          "       More will be arriving soon.       \n";

                    mStages[mStage] = true;
                }

                if (mStages[mStage] && !mUI["TutorialDialog"].Visible)
                {
                    mPaused = false;
                    mStage = "AINurseIntro";

                    for (int i = 0; i < 3; i++)
                    {
                        mNumberOfPatientsSpawned++;
                        mPatients.Add(new Patient());
                        mPatients[mPatients.Count - 1].Position.X = -1.0f;
                        mPatients[mPatients.Count - 1].Position.Z = 32.0f - i * 7;
                        mPatients[mPatients.Count - 1].SpawnPos = mPatients[mPatients.Count - 1].Position;
                        mPatients[mPatients.Count - 1].MacroPathFinder.AutoFollowPath = true;
                        mPatients[mPatients.Count - 1].SetChairs(ref mChairs);
                        mPatients[mPatients.Count - 1].PickAChair();
                        mPatients[mPatients.Count - 1].UniqueID = mNumberOfPatientsSpawned;
                    }

                    mTutTimer = 0;
                }
            }

            if (mStage == "AINurseIntro")
            {
                if (!mStages[mStage])
                {
                    mTutTimer++;
                }

                if (mTutTimer == 300)
                {
                    if (!mStages[mStage])
                    {
                        // Show the dialog
                        mPaused = true;
                        mUI["TutorialDialog"].Visible = true;
                        (mUI["TutorialDialog"] as TutorialDialogUI).Message = " Your job is hard, but you're not alone! \n" +
                                                                              " A team of nurses will assist in shifts. \n" +
                                                                              "   Keep an eye on the bar above to know  \n" +
                                                                              "        when you can summon them.        \n";

                        mStages[mStage] = true;
                    }

                    if (mStages[mStage] && !mUI["TutorialDialog"].Visible)
                    {
                        mPaused = false;
                        mStage = "NurseShiftWaiting";
                        mTutTimer = 0;


                        mAIManager.Enabled = true;
                        //mAIManager.ShiftLength = 2000;
                    }
                }
            }

            if (mStage == "NurseShiftWaiting")
            {
                if (!mStages[mStage] && mAIManager.OnCallAvailable)
                {
                    // Show the dialog
                    mPaused = true;
                    mUI["TutorialDialog"].Visible = true;
                    (mUI["TutorialDialog"] as TutorialDialogUI).Message = "    An on-call nurse is now available.   \n" +
                                                                          "         Press (Y) to summon her.        \n" +
                                                                          " Or wait for additional available nurses \n" +
                                                                          "     As indicated by the icons above.    \n";

                    mStages[mStage] = true;
                }

                if (mStages[mStage] && !mUI["TutorialDialog"].Visible && mAIManager.ShiftActive)
                {
                    mPaused = false;
                    mStage = "NurseShiftBegin";
                    mAIManager.Enabled = true;
                }
            }

            if (mStage == "NurseShiftBegin" && mAIManager.ShiftActive)
            {
                if (!mStages[mStage])
                {
                    mTutTimer++;
                }

                if (mTutTimer == 300)
                {
                    if (!mStages[mStage])
                    {
                        // Show the dialog
                        mPaused = true;
                        mUI["TutorialDialog"].Visible = true;
                        (mUI["TutorialDialog"] as TutorialDialogUI).Message = "  The length of a shift is shown above.  \n" +
                                                                              "    These nurses are highly trained.     \n" +
                                                                              "     They can do everything you can.     \n" +
                                                                              "     Nurses in later levels can not.     \n";

                        mStages[mStage] = true;
                    }

                    if (mStages[mStage] && !mUI["TutorialDialog"].Visible)
                    {
                        mPaused = false;
                        mStage = "NurseShiftEnd";
                        mTutTimer = 0;
                    }
                }
            }

            if (mStage == "NurseShiftEnd")
            {
                if (!mAIManager.ShiftActive && !mStages[mStage])
                {
                    // Show the dialog
                    mPaused = true; 
                    mUI["TutorialDialog"].Visible = true;
                    (mUI["TutorialDialog"] as TutorialDialogUI).Message = "    The on-call nurses' shift is over.   \n" +
                                                                          "    You can call them back again later.  \n" +
                                                                          "The longer you wait, the larger the team.\n";



                    mStages[mStage] = true;
                }

                if (mStages[mStage] && !mUI["TutorialDialog"].Visible)
                {
                    mPaused = false;
                    mStage = "End";
                    mTutTimer = 0;
                }
            }

            if (mStage == "End")
            {
                if (/*Insert emergency shift completion check here*/!mStages[mStage])
                {
                    // Show the dialog
                    mPaused = true;
                    mUI["TutorialDialog"].Visible = true;

                    (mUI["TutorialDialog"] as TutorialDialogUI).Message = "             Congratulations!            \n" +
                                                                          "     You have completed the tutorial.    \n" +
                                                                          "      Now play through a full level.     \n";

                    mStages[mStage] = true;
                }

                if (mStages[mStage] && !mUI["TutorialDialog"].Visible)
                {
                    mPaused = false;
                    mStage = "End";

                    LoadingState loadState = new LoadingState();
                    loadState.StateToLoad = new PlayState();

                    StateManager.SetState(loadState);
                }
            }
        }

        public override void Draw(Renderer renderer, GameTime gameTime)
        {
            // Update UI data
            (mUI["Arrow"] as TutArrowUI).ViewMatrix = renderer.ViewMatrix;
            (mUI["Arrow"] as TutArrowUI).ProjectionMatrix = renderer.ProjectionMatrix;

            base.Draw(renderer, gameTime);
        }
    }
}
